@@ -23,19 +23,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define __DBUFFER_H__
 
 #include "VCommon.h"
+#include "VExcept.h"
 
 namespace vapp {
 
-	/*===========================================================================
-	Output buffer section
+  /*===========================================================================
+  Output buffer section
   ===========================================================================*/
   struct DBufferPart {
     char *pcBuffer;
     int nPtr;
   };
 
-	/*===========================================================================
-	I/O buffer class
+  /*===========================================================================
+  I/O buffer class
   ===========================================================================*/
   class DBuffer {
     public:
@@ -49,18 +50,29 @@ namespace vapp {
       /* Methods */
       void initOutput(int nPartSize);
       void initInput(char *pcInput,int nInputSize);
-      void writeBuf(const char *pcBuf,int nBufSize);    
-      void readBuf(char *pcBuf,int nBufSize);
+      
+      template<typename _ConstIter>
+      void writeBuf(_ConstIter pcBuf,int nBufSize);
+      
+      template<typename _Iter>
+      void readBuf(_Iter pcBuf,int nBufSize);
+      
+      void writeBuf_LE(const char *pcBuf,int nBufSize);    
+      void readBuf_LE(char *pcBuf,int nBufSize);
       int numRemainingBytes(void);
       const char *convertOutputToInput(void);
       
       /* Some I/O */
+      void write(std::string s);
+
+      void read(std::string &s);
+
       template<typename _T> void operator <<(_T n) {
-        writeBuf(reinterpret_cast<const char *>(&n),sizeof(_T));
+        writeBuf_LE(reinterpret_cast<const char *>(&n),sizeof(_T));
       }
       
       template<typename _T> void operator >>(_T &n) {
-        readBuf(reinterpret_cast<char *>(&n),sizeof(_T));
+        readBuf_LE(reinterpret_cast<char *>(&n),sizeof(_T));
       }
       
       /* Data interface */
@@ -88,7 +100,7 @@ namespace vapp {
       void _NewPart(void);
   };
   
-};
+}
 
 #endif
 
