@@ -24,6 +24,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *  As usual, I was shocked by the level of complexity of an editor :)
  */
 #define XMOTO_EDITOR
+#include "BuildConfig.h"
 #include "Editor.h"
 #include "VFileIO.h"
 #include "VBezier.h"
@@ -37,6 +38,21 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 namespace vapp {
 
+  /*============================================================================
+  Editor app constructor
+  ============================================================================*/
+  EditorApp::EditorApp() {
+    m_pLevelSrc=NULL; 
+    m_bLeftButtonDown=false; 
+    m_pEntityToCopy=NULL; 
+    m_bJustPackage=false;
+#if defined(SUPPORT_WEBACCESS)
+    m_themeChoicer = new ThemeChoicer(NULL,NULL);         
+#else
+    m_themeChoicer = new ThemeChoicer();
+#endif
+  }
+  
   /*============================================================================
   Draw graphics
   ============================================================================*/
@@ -273,20 +289,32 @@ namespace vapp {
       m_Log.msg("Loading level from '%s'...",Path.c_str());         
       m_pLevelSrc = new LevelSrc;
       m_pLevelSrc->setFileName( Path );
-      m_pLevelSrc->loadXML();
-
-      resetView();
-      setState(ES_DEFAULT);
+      try {
+        m_pLevelSrc->loadXML();
+        resetView();
+        setState(ES_DEFAULT);
+      }
+      catch(Exception &e) {
+        m_Log.msg("File '%s' failed to load, creating new instead.",m_LevelToInitLoad.c_str());
+        m_Log.msg("If you're sure that the level SHOULD work, DON'T save now, exit instead!");
+        m_Log.msg("(otherwise you might overwrite a valid level)");
+        
+        delete m_pLevelSrc;
+        m_pLevelSrc = NULL;
+        
+        _NewLevel();
+        m_pLevelSrc->setFileName( Path );
+      }
     }        
     else {
       _NewLevel();
     }
     
-    /* Load default theme */
-    m_theme.load("Themes/original.xml");
+    /* Load default theme */    
+    m_theme.load(m_themeChoicer->getFileName(THEME_DEFAULT_THEMENAME));
 
-		/* Output log message */
-		m_Log.msg("Ready. (X-Moto version %s)",getVersionString().c_str());		
+    /* Output log message */
+    m_Log.msg("Ready. (X-Moto version %s)",getVersionString().c_str());		
   }
   
   /*============================================================================
@@ -641,7 +669,7 @@ namespace vapp {
         /* Should we invoke the packager? */
         if(UserArgs[i] == "-pack") {
           Packager::go();
-          exit(1); /* leaks memory, but who cares? :) */
+          exit(0); /* leaks memory, but who cares? :) */
         }
       }
     }
@@ -2487,5 +2515,4 @@ namespace vapp {
     }        
   }
       
-};
-
+}
