@@ -281,6 +281,8 @@ namespace vapp {
       const std::string &getContextHelp(void) {return m_ContextHelp;}
       void clearMsgBoxActive(void) {m_bActiveMsgBox = false; /* pain */}
     
+      bool isUglyMode() {return getApp()->isUglyMode();}
+
     protected:
       /* Protected interface */
       void addChildW(UIWindow *pWindow);
@@ -322,6 +324,7 @@ namespace vapp {
       UIEdit(UIWindow *pParent,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0) {
         initW(pParent,x,y,Caption,nWidth,nHeight);
         m_nCursorPos = 0;
+	m_hideText = false;
       }      
     
       /* Methods */
@@ -329,20 +332,21 @@ namespace vapp {
       virtual bool keyDown(int nKey,int nChar);
       
       virtual bool offerActivation(void) {return true;}
+      void hideText(bool bHideText) {m_hideText=bHideText;} 
 
     private:
       /* Data */
       int m_nCursorPos;
-    
+      bool m_hideText;    
   };
 
 	/*===========================================================================
 	UI frame
   ===========================================================================*/
   enum UIFrameStyle {
-    UI_FRAMESTYLE_MENU,                         /* "Iron-like" scaled menu */
-    UI_FRAMESTYLE_TRANS,                        /* Generic transparent one */
-    UI_FRAMESTYLE_LEFTTAG,                      /* Like above, but with left tag */
+    UI_FRAMESTYLE_MENU,                  /* "Iron-like" scaled menu */
+    UI_FRAMESTYLE_TRANS,                 /* Generic transparent one */
+    UI_FRAMESTYLE_LEFTTAG                /* Like above, but with left tag */
   };
   
   class UIFrame : public UIWindow {
@@ -494,7 +498,7 @@ namespace vapp {
 	Sprite *pSprite;
 	pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "DarkBlob");
 	if(pSprite != NULL) {
-	  m_pDarkBlobTexture = pSprite->getTexture(false,true,false);
+	  m_pDarkBlobTexture = pSprite->getTexture(false,true, FM_NEAREST);
 	}
         
         m_pCustomBackgroundTexture = NULL;
@@ -511,12 +515,12 @@ namespace vapp {
       UIAlign getHAlign(void) {return m_HAlign;}
       void setBackgroundShade(bool b) {m_bBackgroundShade=b;}
       void setBackground(Texture *p) {m_pCustomBackgroundTexture = p;}
-      
+
     private:
       /* Data */
       UIAlign m_VAlign,m_HAlign;
       bool m_bBackgroundShade;
-      
+
       Texture *m_pDarkBlobTexture;
       Texture *m_pCustomBackgroundTexture;
   };
@@ -533,7 +537,7 @@ namespace vapp {
     UI_BUTTON_TYPE_LARGE,
     UI_BUTTON_TYPE_SMALL,
     UI_BUTTON_TYPE_CHECK,
-    UI_BUTTON_TYPE_RADIO,
+    UI_BUTTON_TYPE_RADIO
   };
   
   class UIButton : public UIWindow {
@@ -788,6 +792,6 @@ namespace vapp {
       int _GetActiveIdx(UIRootActCandidate *pMap,int nNum);
   };
 
-};
+}
 
 #endif
