@@ -51,7 +51,7 @@ namespace vapp {
     UIFont *pOldFont = getFont();
     setFont(m_pHFont);
     
-    putText(40,64,GAMETEXT_ALLRECORDS);
+    putText(40,64,std::string(GAMETEXT_ALLRECORDS) + ": ");
     for(int i=0;i<m_Col1.size();i++) {
       if(i == m_nHighlight1) {
         setTextSolidColor(MAKE_COLOR(255,255,0,255));
@@ -61,7 +61,7 @@ namespace vapp {
       setTextSolidColor(MAKE_COLOR(255,255,255,255));
     }
 
-    putText(40,100 + 16*m_Col1.size(),GAMETEXT_PERSONALRECORDS);
+    putText(40,100 + 16*m_Col1.size(),std::string(GAMETEXT_PERSONALRECORDS) + ":");
     for(int i=0;i<m_Col3.size();i++) {
       if(i == m_nHighlight2) {
         setTextSolidColor(MAKE_COLOR(255,255,0,255));
@@ -74,5 +74,5 @@ namespace vapp {
     setFont(pOldFont);
   }
 
-};
+}
 
