@@ -38,15 +38,22 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "WWWAppInterface.h"
 #include "GUIXMoto.h"
 #include "Stats.h"
+#include "Credits.h"
 
+#define PRESTART_ANIMATION_TIME 2.0
+#define PRESTART_ANIMATION_LEVEL_MSG_DURATION 1.0
+#define PRESTART_ANIMATION_MARGIN_SIZE 5
+#define PRESTART_ANIMATION_CURVE 3.0
+  
 namespace vapp {
 
-	/*===========================================================================
-	Overall game states
+  /*===========================================================================
+  Overall game states
   ===========================================================================*/
   enum GameState {
     GS_UNASSIGNED = 0,
     GS_MENU,                  /* In the game menu */
+    GS_PREPLAYING,            /* Just before the game start */
     GS_PLAYING,               /* Playing the game */
     GS_PAUSE,                 /* Paused from GS_PLAYING */
     GS_JUSTDEAD,              /* Head-banging too much */
@@ -55,13 +62,14 @@ namespace vapp {
     GS_REPLAYING,             /* Replaying */
     GS_LEVEL_INFO_VIEWER,     /* In level info viewer */
     GS_LEVELPACK_VIEWER,      /* In level pack viewer */
+    GS_CREDITSMODE,           /* Credits/replay */
 #if defined(SUPPORT_WEBACCESS)
-    GS_EDIT_WEBCONFIG,        /* Editing internet configuration */
+    GS_EDIT_WEBCONFIG         /* Editing internet configuration */
 #endif
   };
 
-	/*===========================================================================
-	Menu background graphical settings
+  /*===========================================================================
+  Menu background graphical settings
   ===========================================================================*/
   enum MenuBackgroundGraphics {
     MENU_GFX_OFF,
@@ -78,8 +86,8 @@ namespace vapp {
   };
 #endif
 
-	/*===========================================================================
-	Level packs
+  /*===========================================================================
+  Level packs
   ===========================================================================*/
   struct LevelPack {
     std::string Name;
@@ -90,8 +98,8 @@ namespace vapp {
     bool bShowWebTimes;
   };
 
-	/*===========================================================================
-	Game application
+  /*===========================================================================
+  Game application
   ===========================================================================*/
   #if defined(SUPPORT_WEBACCESS)
   class GameApp : public App, public WWWAppInterface {
@@ -100,7 +108,8 @@ namespace vapp {
   #endif
     public:
       virtual ~GameApp() {}
-      GameApp() {m_bShowMiniMap=true;
+      GameApp() {m_pCredits = NULL;
+                 m_bShowMiniMap=true;
                  m_bDebugMode=false;
                  m_bListLevels=false;
                  m_bListReplays=false;
@@ -108,6 +117,9 @@ namespace vapp {
                  m_bShowFrameRate=false;
                  m_bEnableLevelCache=true;
                  m_bEnableMenuMusic=false;
+                 m_bEnableInitZoom=false;
+                 m_bCleanCache=false;
+                 m_bEnableDeathAnim=true;
                  m_pQuitMsgBox=NULL;
                  m_pNotifyMsgBox=NULL;
 #if defined(SUPPORT_WEBACCESS)
@@ -126,50 +138,57 @@ namespace vapp {
                  m_pLevelPackViewer=NULL;  
                  m_pActiveLevelPack=NULL;
                  m_pGameInfoWindow=NULL;
+                 m_fFrameTime = 0;
+                 m_fFPS_Rate = 0;
                  m_b50FpsMode = false;
                  m_bUglyMode = false;
+                 m_bTestThemeMode = false;
                  m_pReplay = NULL;
                  m_pMenuMusic = NULL;
 #if defined(ALLOW_GHOST)
-		 m_pGhostReplay = NULL;
-		 m_lastGhostReplay = "";
-		 GhostSearchStrategies[0] = GHOST_STRATEGY_MYBEST;
-		 GhostSearchStrategies[1] = GHOST_STRATEGY_THEBEST;
-		 GhostSearchStrategies[2] = GHOST_STRATEGY_BESTOFROOM;
-		 m_bEnableGhost = true;
-		 m_bShowGhostTimeDiff = true;
-		 m_GhostSearchStrategy = GHOST_STRATEGY_MYBEST;
+     m_pGhostReplay = NULL;
+     m_lastGhostReplay = "";
+     GhostSearchStrategies[0] = GHOST_STRATEGY_MYBEST;
+     GhostSearchStrategies[1] = GHOST_STRATEGY_THEBEST;
+     GhostSearchStrategies[2] = GHOST_STRATEGY_BESTOFROOM;
+     m_bEnableGhost = true;
+     m_bShowGhostTimeDiff = true;
+     m_GhostSearchStrategy = GHOST_STRATEGY_MYBEST;
                  m_bEnableGhostInfo = false;
                  m_bGhostMotionBlur = true;
 #endif
-		 m_bAutosaveHighscoreReplays = true;
+     m_bAutosaveHighscoreReplays = true;
                  m_bRecordReplays = true;
                  m_bShowCursor = true;
                  m_bEnableEngineSound = true;
                  m_bCompressReplays = true;
                  m_bBenchmark = false;
-                 m_bEnableContextHelp = true;                 
+                 m_bEnableContextHelp = true;     
 
 #if defined(SUPPORT_WEBACCESS)
                  m_bShowWebHighscoreInGame = false;
                  m_bEnableWebHighscores = true;
-		 m_pWebHighscores = NULL;
-		 m_pWebLevels = NULL;
-		 m_fDownloadTaskProgressLast = 0;
-		 m_bWebHighscoresUpdatedThisSession = false;
-		 m_bWebLevelsToDownload = false;
+     m_pWebHighscores = NULL;
+     m_pWebLevels = NULL;
+     m_pWebRooms = NULL;
+     m_fDownloadTaskProgressLast = 0;
+     m_bWebHighscoresUpdatedThisSession = false;
+     m_bWebLevelsToDownload = false;
 
-		 m_bEnableCheckNewLevelsAtStartup  = true;
-		 m_bEnableCheckHighscoresAtStartup = true;
+     m_bEnableCheckNewLevelsAtStartup  = true;
+     m_bEnableCheckHighscoresAtStartup = true;
 #endif
-		 m_fLastSqueekTime = 0.0f;
+     m_fLastSqueekTime = 0.0f;
 
 
-		 m_Renderer.setTheme(&m_theme);
-		 m_MotoGame.setRenderer(&m_Renderer);
+     m_Renderer.setTheme(&m_theme);
+     m_MotoGame.setRenderer(&m_Renderer);
 
-		 m_currentThemeName = THEME_DEFAULT_THEMENAME;
-                 }
+     m_currentThemeName = THEME_DEFAULT_THEMENAME;
+
+     m_bPrePlayAnim = true;
+     m_bShowEngineCounter = true;
+      }
                  
 #if defined(SUPPORT_WEBACCESS)                 
         /* WWWAppInterface implementation */ 
@@ -182,7 +201,6 @@ namespace vapp {
         virtual bool shouldLevelBeUpdated(const std::string &LevelID);
 
         virtual std::string levelPathForUpdate(const std::string &p_LevelId);
-        virtual std::string levelCRC32Sum(const std::string &p_LevelId);
         virtual std::string levelMD5Sum(const std::string &LevelID);
 
 #endif
@@ -205,9 +223,11 @@ namespace vapp {
       /* Methods */
       void setState(GameState s);
       void notifyMsg(std::string Msg);      
+      void setPrePlayAnim(bool pEnabled);
       
       /* Data interface */
-      bool isUglyMode(void) {return m_bUglyMode;}
+      bool isUglyMode() {return m_bUglyMode;}
+      bool isTestThemeMode(void) {return m_bTestThemeMode;}
     
     private: 
       EngineSoundSimulator m_EngineSound;
@@ -216,6 +236,8 @@ namespace vapp {
       ReplayList m_ReplayList;                  /* Replay list */
       std::vector<LevelPack *> m_LevelPacks;    /* Level packs */
 
+      bool m_bEnableInitZoom;                   /* true: Perform initial level scroll/zoom */
+      bool m_bEnableDeathAnim;                  /* true: Bike falls apart at when dead */
       bool m_bEnableMenuMusic;                  /* true: Play menu music */      
       bool m_bEnableContextHelp;                /* true: Show context help */
       bool m_bBenchmark;                        /* true: Test game performance */
@@ -224,7 +246,11 @@ namespace vapp {
       bool m_bListReplays;                      /* true: list replays */
       bool m_bTimeDemo;                         /* true: (valid for replaying) - performance benchmark */
       bool m_bDebugMode;                        /* true: show debug info */
-      bool m_bUglyMode;													/* true: fast 'n ugly graphics */
+      bool m_bUglyMode;                         /* true: fast 'n ugly graphics */
+      bool m_bCleanCache;                       /* true: clean the level cache at startup */
+      bool m_bShowEngineCounter;
+
+      bool m_bTestThemeMode;
       bool m_bEnableEngineSound;                /* true: engine sound is enabled */
       bool m_bCompressReplays;                  /* true: compress replays with zlib */
       bool m_bEnableLevelCache;                 /* true: cache levels for faster loading */
@@ -275,6 +301,7 @@ namespace vapp {
 #if defined(SUPPORT_WEBACCESS)
       bool m_bShowWebHighscoreInGame;           /* true: Show world highscore inside the game */
       WebRoom *m_pWebHighscores;
+      WebRooms *m_pWebRooms;
       WebLevels *m_pWebLevels;
       ProxySettings m_ProxySettings;
       std::string m_DownloadingInformation;
@@ -289,8 +316,26 @@ namespace vapp {
 
 #endif
       
+      bool m_bCreditsModeActive;
+      
       ThemeChoicer *m_themeChoicer;
 
+      float m_fPrePlayStartTime;
+      bool m_bPrePlayAnim;
+      float m_fPrePlayCameraLastX;
+      float m_fPrePlayCameraLastY;
+      float m_fPrePlayStartInitZoom;
+      float m_fPrePlayStartCameraX;
+      float m_fPrePlayStartCameraY;
+      float m_fPreCameraStartX;
+      float m_fPreCameraStartY;
+      float m_fPreCameraFinalX;
+      float m_fPreCameraFinalY;
+      float m_zoomX;
+      float m_zoomY;
+      float m_zoomU;
+      float static_time;
+
       Stats m_GameStats;
       UIWindow *m_pStatsReport;
       
@@ -394,7 +439,14 @@ namespace vapp {
       bool m_bRecordReplays;
       float m_fReplayFrameRate;
       float m_fCurrentReplayFrameRate;
-      std::string m_currentThemeName;            
+      std::string m_currentThemeName; 
+      
+      /* Credits */
+      Credits *m_pCredits;         
+      
+      /* Main loop statics */
+      double m_fFrameTime;
+      float m_fFPS_Rate;
 
       /* Helpers */
 #if defined(SUPPORT_WEBACCESS) 
@@ -414,6 +466,9 @@ namespace vapp {
       void _CreateLevelLists(UILevelList *pExternalLevels,UILevelList *pInternalLevels);
       void _CreateReplaysList(UIList *pList);
       void _CreateThemesList(UIList *pList);
+#if defined(SUPPORT_WEBACCESS) 
+      void _CreateRoomsList(UIList *pList);
+#endif
       void _CreateProfileList(void);
       void _CreateDefaultConfig(void);
       void _CreateLevelPackLevelList(void);
@@ -435,6 +490,9 @@ namespace vapp {
       void _UpdateLevelLists(void);
       void _UpdateReplaysList(void);
       void _UpdateThemesLists(void);
+#if defined(SUPPORT_WEBACCESS) 
+      void _UpdateRoomsLists(void);
+#endif
       void _GameScreenshot(void);
       void _SaveReplay(const std::string &Name);
     
@@ -462,7 +520,10 @@ namespace vapp {
       void _UpdateWebThemes(bool bSilent);
       void _UpdateWebTheme(ThemeChoice* pThemeChoice, bool bNotify);
       void _UpgradeWebHighscores();
+      void _UpdateWebRooms(bool bSilent);
+      void _UpgradeWebRooms(bool bUpdateMenus);
       void _DownloadExtraLevels(void);
+      void _UploadHighscore(std::string p_replayname);
       void _ConfigureProxy(void);
 #endif
 
@@ -473,11 +534,35 @@ namespace vapp {
 
 #if defined(ALLOW_GHOST) 
       std::string _getGhostReplayPath(std::string p_levelId,
-				      GhostSearchStrategy p_strategy);
+              GhostSearchStrategy p_strategy);
 #endif
 
+      void switchUglyMode(bool bUgly);
+      void switchTestThemeMode(bool mode);
+
+      void statePrestart_init();
+      void statePrestart_step();
+      void prestartAnimation_init();
+      void prestartAnimation_step();
+      
+      std::string splitText(const std::string &str, int p_breakLineLength);
+      
+      /* Main loop utility functions */
+      void _UpdateFPSCounter(void);
+      void _PrepareFrame(void);
+      void _PreUpdateGUI(void);
+      void _PreUpdateMenu(void);
+      void _DrawMainGUI(void);
+      void _DrawMouseCursor(void);
+      int _UpdateGamePlaying(void); /* returns number of physics steps performed */
+      int _UpdateGameReplaying(void); /* return whether the game state is valid */
+      void _PostUpdatePlaying(void);
+      void _PostUpdatePause(void);
+      void _PostUpdateJustDead(void);
+      void _PostUpdateFinished(void);
+
   };
 
-};
+}
 
 #endif
