@@ -126,7 +126,7 @@ namespace vapp {
     else {
       m_nStillFrames=0;
     }
-    bool bSleep = false;
+    bool bSleep = false;        
     
     //printf("{%d}\n",m_Collision.isDynamicTouched());
     //printf("]",m_nStillFrames);        
@@ -150,9 +150,11 @@ namespace vapp {
       Vector2f Fqv = Fq - m_BikeS.PrevFq;
       Vector2f FSpring = Fq * PHYS_SUSP_SPRING; 
       Vector2f FDamp = Fqv * PHYS_SUSP_DAMP;
-      Vector2f FTotal = FSpring + FDamp;  
-      dBodyAddForce(m_FrontWheelBodyID,FTotal.x,FTotal.y,0);
-      dBodyAddForceAtPos(m_FrameBodyID,-FTotal.x,-FTotal.y,0,m_BikeS.RFrontWheelP.x,m_BikeS.RFrontWheelP.y,0);
+      Vector2f FTotal = FSpring + FDamp; 
+      if(m_bodyDetach == false || m_BikeS.Dir == DD_LEFT) { 
+	dBodyAddForce(m_FrontWheelBodyID,FTotal.x,FTotal.y,0);
+	dBodyAddForceAtPos(m_FrameBodyID,-FTotal.x,-FTotal.y,0,m_BikeS.RFrontWheelP.x,m_BikeS.RFrontWheelP.y,0);
+      }
       m_BikeS.PrevFq = Fq;
 
       /* Update rear suspension */
@@ -161,8 +163,10 @@ namespace vapp {
       Vector2f RSpring = Rq * PHYS_SUSP_SPRING; 
       Vector2f RDamp = Rqv * PHYS_SUSP_DAMP;
       Vector2f RTotal = RSpring + RDamp;   
-      dBodyAddForce(m_RearWheelBodyID,RTotal.x,RTotal.y,0);
-      dBodyAddForceAtPos(m_FrameBodyID,-RTotal.x,-RTotal.y,0,m_BikeS.RRearWheelP.x,m_BikeS.RRearWheelP.y,0);
+      if(m_bodyDetach == false || m_BikeS.Dir == DD_RIGHT) {
+	dBodyAddForce(m_RearWheelBodyID,RTotal.x,RTotal.y,0);
+	dBodyAddForceAtPos(m_FrameBodyID,-RTotal.x,-RTotal.y,0,m_BikeS.RRearWheelP.x,m_BikeS.RRearWheelP.y,0);
+      }
       m_BikeS.PrevRq = Rq;
       
       /* Have any of the suspensions reached the "squeek-point"? (rate of compression/decompression at 
@@ -296,6 +300,17 @@ namespace vapp {
     nSlipFrameInterlace++;
         
     nNumContacts = _IntersectWheelLevel( m_BikeS.FrontWheelP,m_BikeP.WR,Contacts );
+    if(nNumContacts>0) {
+      if(bFrontWheelTouching == false) {
+	bFrontWheelTouching = true;
+	scriptCallVoidNumberArg("OnWheel1Touchs", 1);
+      }
+    } else {
+      if(bFrontWheelTouching) {
+	bFrontWheelTouching = false;
+	scriptCallVoidNumberArg("OnWheel1Touchs", 0);
+      }
+    }
     if(m_Collision.isDynamicTouched()) {
       if(!dBodyIsEnabled(m_FrontWheelBodyID)) dBodyEnable(m_FrontWheelBodyID);
       if(!dBodyIsEnabled(m_RearWheelBodyID)) dBodyEnable(m_RearWheelBodyID);
@@ -343,6 +358,18 @@ namespace vapp {
     }
 
     nNumContacts = _IntersectWheelLevel( m_BikeS.RearWheelP,m_BikeP.WR,Contacts );
+    if(nNumContacts>0) {
+      if(bRearWheelTouching == false) {
+	bRearWheelTouching = true;
+	scriptCallVoidNumberArg("OnWheel2Touchs", 1);
+      }
+    } else {
+      if(bRearWheelTouching) {
+	bRearWheelTouching = false;
+	scriptCallVoidNumberArg("OnWheel2Touchs", 0);
+      }
+    }
+
     if(m_Collision.isDynamicTouched()) {
       if(!dBodyIsEnabled(m_FrontWheelBodyID)) dBodyEnable(m_FrontWheelBodyID);
       if(!dBodyIsEnabled(m_RearWheelBodyID)) dBodyEnable(m_RearWheelBodyID);
@@ -389,22 +416,147 @@ namespace vapp {
       }
     }        
     
+    /* body */
+    if(m_bodyDetach) {
+
+      /*
+      // m_PlayerTorsoBody
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideLine(m_BikeS.ShoulderP.x, m_BikeS.ShoulderP.y,
+					       m_BikeS.LowerBodyP.x, m_BikeS.LowerBodyP.y,
+					       Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideLine(m_BikeS.Shoulder2P.x,  m_BikeS.Shoulder2P.y,
+					       m_BikeS.LowerBody2P.x, m_BikeS.LowerBody2P.y,
+					       Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerTorsoBodyID : m_PlayerTorsoBodyID2, 0);           
+      }
+
+      // m_PlayerLArmBodyID
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideLine(m_BikeS.ElbowP.x, m_BikeS.ElbowP.y,
+					       m_BikeS.HandP.x, m_BikeS.HandP.y,
+					       Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideLine(m_BikeS.Elbow2P.x,  m_BikeS.Elbow2P.y,
+					       m_BikeS.Hand2P.x, m_BikeS.Hand2P.y,
+					       Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerLArmBodyID : m_PlayerLArmBodyID2, 0);           
+      }
+
+      // m_PlayerUArmBodyID
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideLine(m_BikeS.ElbowP.x, m_BikeS.ElbowP.y,
+					       m_BikeS.ShoulderP.x, m_BikeS.ShoulderP.y,
+					       Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideLine(m_BikeS.Elbow2P.x,  m_BikeS.Elbow2P.y,
+					       m_BikeS.Shoulder2P.x, m_BikeS.Shoulder2P.y,
+					       Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerUArmBodyID : m_PlayerUArmBodyID2, 0);           
+      }  
+
+      // m_PlayerULegBodyID
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideLine(m_BikeS.LowerBodyP.x, m_BikeS.LowerBodyP.y,
+					       m_BikeS.KneeP.x, m_BikeS.KneeP.y,
+					       Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideLine(m_BikeS.LowerBody2P.x,  m_BikeS.LowerBody2P.y,
+					       m_BikeS.Knee2P.x, m_BikeS.Knee2P.y,
+					       Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerULegBodyID : m_PlayerULegBodyID2, 0);           
+      } 
+
+      // m_PlayerLLegBodyID
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideLine(m_BikeS.KneeP.x, m_BikeS.KneeP.y,
+					       m_BikeS.FootP.x, m_BikeS.FootP.y,
+					       Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideLine(m_BikeS.Knee2P.x,  m_BikeS.Knee2P.y,
+					       m_BikeS.Foot2P.x, m_BikeS.Foot2P.y,
+					       Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerLLegBodyID : m_PlayerLLegBodyID2, 0);           
+      }
+      */
+
+      /*
+      // hand
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideCircle(m_BikeS.HandP.x, m_BikeS.HandP.y, 0.1, Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideCircle(m_BikeS.Hand2P.x, m_BikeS.Hand2P.y, 0.1, Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerHandAnchorBodyID : m_PlayerHandAnchorBodyID2, 0);           
+      }
+
+      // foot
+      if(m_BikeS.Dir == DD_RIGHT) {
+	nNumContacts = m_Collision.collideCircle(m_BikeS.FootP.x, m_BikeS.FootP.y, 0.1, Contacts, 100);
+      } else {
+	nNumContacts = m_Collision.collideCircle(m_BikeS.Foot2P.x, m_BikeS.Foot2P.y, 0.1, Contacts, 100);
+      }
+      for(int i=0;i<nNumContacts;i++) {
+        dJointAttach(dJointCreateContact(m_WorldID,
+      					 m_ContactGroup,
+      					 &Contacts[i]),
+		     m_BikeS.Dir == DD_RIGHT ?
+		     m_PlayerFootAnchorBodyID : m_PlayerFootAnchorBodyID2 , 0);           
+      }
+      */
+
+    }
+
     /* Player head */
     if(m_BikeS.Dir == DD_RIGHT) {
-      if(_IntersectHeadLevel(m_BikeS.HeadP,m_BikeP.fHeadSize,m_PrevHeadP)) {
-				GameEvent *pEvent = createGameEvent(GAME_EVENT_PLAYER_DIES);
-				if(pEvent != NULL) {
-					pEvent->u.PlayerDies.bWrecker = false;
-				}
+      if(_IntersectHeadLevel(m_BikeS.HeadP,m_BikeP.fHeadSize,m_PrevActiveHead)) {
+	createGameEvent(new MGE_PlayerDies(getTime(), false));
       }
+      
+      m_PrevActiveHead = m_BikeS.HeadP;
     }
     else if(m_BikeS.Dir == DD_LEFT) {
-      if(_IntersectHeadLevel(m_BikeS.Head2P,m_BikeP.fHeadSize,m_PrevHead2P)) {
-				GameEvent *pEvent = createGameEvent(GAME_EVENT_PLAYER_DIES);
-				if(pEvent != NULL) {
-					pEvent->u.PlayerDies.bWrecker = false;
-				}
+      if(_IntersectHeadLevel(m_BikeS.Head2P,m_BikeP.fHeadSize,m_PrevActiveHead)) {
+	createGameEvent(new MGE_PlayerDies(getTime(), false));
       }
+
+      m_PrevActiveHead = m_BikeS.Head2P;
     }
         
     //m_PrevFrontWheelP = m_BikeS.FrontWheelP;
@@ -422,16 +574,21 @@ namespace vapp {
     Vector2f PFqv = PFq - m_BikeS.PrevPFq;
     Vector2f PFSpring = PFq * PHYS_RIDER_SPRING; 
     Vector2f PFDamp = PFqv * PHYS_RIDER_DAMP;
-    Vector2f PFTotal = PFSpring + PFDamp;  
-    dBodyAddForce(m_PlayerFootAnchorBodyID,PFTotal.x,PFTotal.y,0);
+    Vector2f PFTotal = PFSpring + PFDamp;
+    if(m_bodyDetach == false) {
+      dBodyAddForce(m_PlayerFootAnchorBodyID,PFTotal.x,PFTotal.y,0);
+    }
+
     m_BikeS.PrevPFq = PFq;    
            
     Vector2f PHq = HandRP - m_BikeS.HandP;
     Vector2f PHqv = PHq - m_BikeS.PrevPHq;
     Vector2f PHSpring = PHq * PHYS_RIDER_SPRING; 
     Vector2f PHDamp = PHqv * PHYS_RIDER_DAMP;
-    Vector2f PHTotal = PHSpring + PHDamp;  
-    dBodyAddForce(m_PlayerHandAnchorBodyID,PHTotal.x,PHTotal.y,0);
+    Vector2f PHTotal = PHSpring + PHDamp;
+    if(m_bodyDetach == false) {
+      dBodyAddForce(m_PlayerHandAnchorBodyID,PHTotal.x,PHTotal.y,0);
+    }
     m_BikeS.PrevPHq = PHq;    
 
     FootRP = m_BikeS.WantedFoot2P;
@@ -442,7 +599,9 @@ namespace vapp {
     PFSpring = PFq * PHYS_RIDER_SPRING; 
     PFDamp = PFqv * PHYS_RIDER_DAMP;
     PFTotal = PFSpring + PFDamp;  
-    dBodyAddForce(m_PlayerFootAnchorBodyID2,PFTotal.x,PFTotal.y,0);
+    if(m_bodyDetach == false) {
+      dBodyAddForce(m_PlayerFootAnchorBodyID2,PFTotal.x,PFTotal.y,0);
+    }
     m_BikeS.PrevPFq2 = PFq;    
            
     PHq = HandRP - m_BikeS.Hand2P;
@@ -450,7 +609,9 @@ namespace vapp {
     PHSpring = PHq * PHYS_RIDER_SPRING; 
     PHDamp = PHqv * PHYS_RIDER_DAMP;
     PHTotal = PHSpring + PHDamp;  
-    dBodyAddForce(m_PlayerHandAnchorBodyID2,PHTotal.x,PHTotal.y,0);
+    if(m_bodyDetach == false) {
+      dBodyAddForce(m_PlayerHandAnchorBodyID2,PHTotal.x,PHTotal.y,0);
+    }
     m_BikeS.PrevPHq2 = PHq;    
        
     /* Perform world simulation step */
@@ -635,7 +796,7 @@ namespace vapp {
     m_BikeS.WantedHand2P.y = m_BikeA.PHp2.x*m_BikeS.fFrameRot[2] + m_BikeA.PHp2.y*m_BikeS.fFrameRot[3] + m_BikeS.CenterP.y;    
 
     /* Still a replay question... */
-    if(pReplayState == NULL) {            
+    if(pReplayState == NULL) {
       dVector3 T;
       
       dJointGetHingeAnchor(m_HandHingeID,T);
@@ -713,7 +874,6 @@ namespace vapp {
     }
           
     Vector2f V;      
-        
     if(bUpdateRider) {        
       /* Calculate head position */
       V = (m_BikeS.ShoulderP - m_BikeS.LowerBodyP);
@@ -841,7 +1001,7 @@ namespace vapp {
     m_KneeHingeID = dJointCreateHinge(m_WorldID,0);
     dJointAttach(m_KneeHingeID,m_PlayerLLegBodyID,m_PlayerULegBodyID);
     dJointSetHingeAnchor(m_KneeHingeID,StartPos.x + m_BikeP.PKVx,StartPos.y + m_BikeP.PKVy,0.0f);
-    dJointSetHingeAxis(m_KneeHingeID,0,0,1);       
+    dJointSetHingeAxis(m_KneeHingeID,0,0,1);
     dJointSetHingeParam(m_KneeHingeID,dParamLoStop,0);
     dJointSetHingeParam(m_KneeHingeID,dParamHiStop,0);
     dJointSetHingeParam(m_KneeHingeID,dParamStopERP,fERP);
@@ -999,7 +1159,7 @@ namespace vapp {
       dContact c[100];
       int nNumContacts = m_Collision.collideLine(LastCp.x,LastCp.y,Cp.x,Cp.y,c,100);
       if(nNumContacts > 0) {
-        return true;
+	return true;
       }
     }
     
@@ -1016,4 +1176,4 @@ namespace vapp {
     return nNumContacts;
   }
   
-};
+}
