@@ -22,6 +22,24 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #ifndef __MOTOGAME_H__
 #define __MOTOGAME_H__
 
+#define MOTOGAME_DEFAULT_GAME_MESSAGE_DURATION 5.0
+
+namespace vapp {
+  /*===========================================================================
+    Entity object types
+    ===========================================================================*/
+  enum EntityType {
+    ET_UNASSIGNED,
+    ET_SPRITE,
+    ET_PLAYERSTART,
+    ET_ENDOFLEVEL,
+    ET_WRECKER,
+    ET_STRAWBERRY,
+    ET_PARTICLESOURCE,
+    ET_DUMMY
+  };
+}
+
 #include "VCommon.h"
 #include "VApp.h"
 #include "VMath.h"
@@ -31,9 +49,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Collision.h"
 #include "ScriptDynamicObjects.h"
 #include "SomersaultCounter.h"
+#include "GameEvents.h"
 
 namespace vapp {
 
+  class MotoGameEvent;
   class Replay;
   class GameRenderer;
 
@@ -60,7 +80,8 @@ namespace vapp {
     EE_GRASS,
     EE_REDBRICKS,
     EE_GRAYBRICKS,
-    EE_BLUEBRICKS
+    EE_BLUEBRICKS,
+    EE_GRASSALT
   };
 
   /*===========================================================================
@@ -117,6 +138,7 @@ namespace vapp {
     LevelBlock *pSrcBlock;                        /* Source block */
     float fRotation;                              /* Block rotation */    
     Vector2f Position;                            /* Block position */
+    bool bBackground;                             /* Background block */
     
     std::vector<Line *> CollisionLines;           /* Line to collide against */
   };
@@ -130,7 +152,7 @@ namespace vapp {
     }
   
     Vector2f P1,P2;                               /* Point-to-point */
-    EdgeEffect Effect;                            /* What? */
+    std::string Effect;                            /* What? */
     LevelBlock *pSrcBlock;                        /* Source block */
   };
 
@@ -215,12 +237,12 @@ namespace vapp {
         
     unsigned char cBikeEngineRPM;     /* Maps to a float between 400 and 5000 */
     
-    char cRearWheelX,cRearWheelY;     /* Rear wheel position */
-    char cFrontWheelX,cFrontWheelY;   /* Front wheel position */
-    char cElbowX,cElbowY;             /* Elbow position */
-    char cShoulderX,cShoulderY;       /* Shoulder position */
-    char cLowerBodyX,cLowerBodyY;     /* Ass position */
-    char cKneeX,cKneeY;               /* Knee position */
+    signed char cRearWheelX,cRearWheelY;     /* Rear wheel position */
+    signed char cFrontWheelX,cFrontWheelY;   /* Front wheel position */
+    signed char cElbowX,cElbowY;             /* Elbow position */
+    signed char cShoulderX,cShoulderY;       /* Shoulder position */
+    signed char cLowerBodyX,cLowerBodyY;     /* Ass position */
+    signed char cKneeX,cKneeY;               /* Knee position */
   };
 
   /*===========================================================================
@@ -312,23 +334,9 @@ namespace vapp {
     /* Driving */
     float fCurBrake,fCurEngine;    
   };
-  
-	/*===========================================================================
-	Entity object types
-  ===========================================================================*/
-  enum EntityType {
-    ET_UNASSIGNED,
-    ET_SPRITE,
-    ET_PLAYERSTART,
-    ET_ENDOFLEVEL,
-    ET_WRECKER,
-    ET_STRAWBERRY,
-    ET_PARTICLESOURCE,
-    ET_DUMMY
-  };
 
-	/*===========================================================================
-	Arrow pointer
+  /*===========================================================================
+  Arrow pointer
   ===========================================================================*/
   struct ArrowPointer {
     ArrowPointer() {
@@ -340,8 +348,8 @@ namespace vapp {
     float fArrowPointerAngle;
   };
   
-	/*===========================================================================
-	Entity object
+  /*===========================================================================
+  Entity object
   ===========================================================================*/
   struct Entity {
     Entity() {
@@ -380,16 +388,16 @@ namespace vapp {
     /* ET_DUMMY */
   };
 
-	/*===========================================================================
-	Requested player state
+  /*===========================================================================
+  Requested player state
   ===========================================================================*/
   struct GameReqPlayerPos {
     Vector2f Pos;
     bool bDriveRight;  
   };
 
-	/*===========================================================================
-	Game message
+  /*===========================================================================
+  Game message
   ===========================================================================*/
   struct GameMessage {
     bool bNew;                        /* Uninitialized */
@@ -399,340 +407,92 @@ namespace vapp {
     int nAlpha;                       /* Alpha amount */
     bool bOnce;                       /* Unique message */
   };  
-
-	/*===========================================================================
-	Game event types
-  ===========================================================================*/
-  /* never change the order of these values ! 
-     ALWAYS ADD VALUES at the end : (otherwise, old replays will not work !)
-  */
-  enum GameEventType {
-    GAME_EVENT_PLAYER_DIES,
-    GAME_EVENT_PLAYER_ENTERS_ZONE,
-    GAME_EVENT_PLAYER_LEAVES_ZONE,
-    GAME_EVENT_PLAYER_TOUCHES_ENTITY,
-    GAME_EVENT_ENTITY_DESTROYED,
-    
-    GAME_EVENT_LUA_CALL_CLEARMESSAGES,
-    GAME_EVENT_LUA_CALL_PLACEINGAMEARROW,
-    GAME_EVENT_LUA_CALL_PLACESCREENARROW,
-    GAME_EVENT_LUA_CALL_HIDEARROW,
-    GAME_EVENT_LUA_CALL_MESSAGE,
-    GAME_EVENT_LUA_CALL_MOVEBLOCK,
-    GAME_EVENT_LUA_CALL_SETBLOCKPOS,
-    GAME_EVENT_LUA_CALL_SETGRAVITY,
-    GAME_EVENT_LUA_CALL_SETPLAYERPOSITION,
-    GAME_EVENT_LUA_CALL_SETENTITYPOS,
-    GAME_EVENT_LUA_CALL_SETBLOCKCENTER,
-    GAME_EVENT_LUA_CALL_SETBLOCKROTATION,
-
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYROTATION,
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYTRANSLATION,
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYNONE,
-
-    GAME_EVENT_LUA_CALL_CAMERAZOOM,
-    GAME_EVENT_LUA_CALL_CAMERAMOVE,
-
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKROTATION,
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKTRANSLATION,
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKNONE
-  };
-  
-	/*===========================================================================
-	Game event
-  ===========================================================================*/
-  struct GameEventPlayerDies {
-  	bool bWrecker;								/* Killed by wrecker */					
-  };
-  
-  struct GameEventEntityDestroyed {
-		/* Have enough information so that we can recreate the entity */
-		char cEntityID[64];					  /* ID of entity */
-		EntityType Type;              /* Type */
-		float fSize;									/* Size of it */
-		float fPosX,fPosY;					  /* Position of it */
-  };
-  
-  struct GameEventPlayerEntersZone {
-		LevelZone *pZone;							/* Pointer to zone */
-  };
-  
-  struct GameEventPlayerLeavesZone {
-		LevelZone *pZone;							/* Pointer to zone */
-  };
-  
-  struct GameEventPlayerTouchesEntity {
-		char cEntityID[64];					  /* ID of entity */
-		bool bHead;										/* Touched with head? */
-  };
-
-
-  struct GameEventLuaCallClearmessages {
-    /* nothing */
-  };
-  
-  struct GameEventLuaCallPlaceingamearrow {
-    float x, y, angle;
-  };
-  
-  struct GameEventLuaCallPlacescreenarrow {
-    float x, y, angle;
-  };
-  
-  struct GameEventLuaCallHidearrow {
-  };
-  
-  struct GameEventLuaCallMessage {
-    char cMessage[512]; /* message */
-  };
-  
-  struct GameEventLuaCallMoveblock {
-    char  cBlockID[64]; /* ID of block */
-    float x, y;
-  };
-  
-  struct GameEventLuaCallSetblockpos {
-    char  cBlockID[64]; /* ID of block */
-    float x, y;
-  };
-  
-  struct GameEventLuaCallSetgravity {
-    float x, y;
-  };
-  
-  struct GameEventLuaCallSetplayerposition {
-    float x, y;
-    bool bRight;
-  };
-  
-  struct GameEventLuaCallSetentitypos {
-    char  cEntityID[64]; /* ID of entity */
-    float x,y;
-  };
-  
-  struct GameEventLuaCallSetBlockCenter {
-    char  cBlockID[64]; /* ID of block */
-    float x,y;
-  };
-  
-  struct GameEventLuaCallSetBlockRotation {
-    char  cBlockID[64]; /* ID of block */
-    float fAngle;
-  };
-  
-  struct GameEventLuaCallSetDynamicEntityRotation {
-    char  cEntityID[64]; /* ID of entity */
-    float fInitAngle;
-    float fRadius;
-    float fPeriod;
-    int   startTime;
-    int   endTime;
-  };
-
-  struct GameEventLuaCallSetDynamicEntityTranslation {
-    char  cEntityID[64]; /* ID of entity */
-    float fX;
-    float fY;
-    float fPeriod;
-    int   startTime;
-    int   endTime;
-  };
- 
-  struct GameEventLuaCallSetDynamicEntityNone {
-    char cEntityID[64]; /* ID of entity */
-  };
-
-  struct GameEventLuaCallSetDynamicBlockRotation {
-    char  cBlockID[64]; /* ID of block */
-    float fInitAngle;
-    float fRadius;
-    float fPeriod;
-    int   startTime;
-    int   endTime;
-  };
-
-  struct GameEventLuaCallSetDynamicBlockTranslation {
-    char  cBlockID[64]; /* ID of block */
-    float fX;
-    float fY;
-    float fPeriod;
-    int   startTime;
-    int   endTime;
-  };
-
-  struct GameEventLuaCallSetDynamicBlockNone {
-    char cBlockID[64]; /* ID of block */
-  };
-
-  struct GameEventLuaCallCameraZoom {
-    float fZoom;
-  };
-
-  struct GameEventLuaCallCameraMove {
-    float x, y;
-  };
-
-
-  struct GameEvent {
-    int nSeq;                         /* Sequence number */
-    GameEventType Type;								/* Type of event */
-    
-    union GameEvent_u {								
-      /* GAME_EVENT_PLAYER_DIES */		
-      GameEventPlayerDies PlayerDies;								
-      
-      /* GAME_EVENT_ENTITY_DESTROYED */
-      GameEventEntityDestroyed EntityDestroyed;
-      
-      /* GAME_EVENT_PLAYER_ENTERS_ZONE */
-      GameEventPlayerEntersZone PlayerEntersZone;
-      
-      /* GAME_EVENT_PLAYER_LEAVES_ZONE */
-      GameEventPlayerLeavesZone PlayerLeavesZone;
-      
-      /* GAME_EVENT_PLAYER_TOUCHES_ENTITY */
-      GameEventPlayerTouchesEntity PlayerTouchesEntity;
-
-      /* GAME_EVENT_LUA_CALL_CLEARMESSAGES     */
-      GameEventLuaCallClearmessages LuaCallClearmessages;
-
-      /* GAME_EVENT_LUA_CALL_PLACEINGAMEARROW  */
-      GameEventLuaCallPlaceingamearrow LuaCallPlaceingamearrow;
-
-      /* GAME_EVENT_LUA_CALL_PLACESCREENARROW  */
-      GameEventLuaCallPlacescreenarrow LuaCallPlacescreenarrow;
-
-      /* GAME_EVENT_LUA_CALL_HIDEARROW	 */
-      GameEventLuaCallHidearrow LuaCallHidearrow;
-
-      /* GAME_EVENT_LUA_CALL_MESSAGE		 */
-      GameEventLuaCallMessage LuaCallMessage;
-
-      /* GAME_EVENT_LUA_CALL_MOVEBLOCK	 */
-      GameEventLuaCallMoveblock LuaCallMoveblock;
-
-      /* GAME_EVENT_LUA_CALL_SETBLOCKPOS	 */
-      GameEventLuaCallSetblockpos LuaCallSetblockpos;
-
-      /* GAME_EVENT_LUA_CALL_SETGRAVITY	 */
-      GameEventLuaCallSetgravity LuaCallSetgravity;
-
-      /* GAME_EVENT_LUA_CALL_SETPLAYERPOSITION */
-      GameEventLuaCallSetplayerposition LuaCallSetplayerposition;
-
-      /* GAME_EVENT_LUA_CALL_SETENTITYPOS      */
-      GameEventLuaCallSetentitypos LuaCallSetentitypos;
-      
-      /* GAME_EVENT_LUA_CALL_SETBLOCKCENTER */
-      GameEventLuaCallSetBlockCenter LuaCallSetBlockCenter;
-
-      /* GAME_EVENT_LUA_CALL_SETBLOCKROTATION */
-      GameEventLuaCallSetBlockRotation LuaCallSetBlockRotation;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICENTITYROTATION */
-      GameEventLuaCallSetDynamicEntityRotation LuaCallSetDynamicEntityRotation;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICENTITYROTATION */
-      GameEventLuaCallSetDynamicEntityTranslation LuaCallSetDynamicEntityTranslation;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICENTITYNONE */
-      GameEventLuaCallSetDynamicEntityNone LuaCallSetDynamicEntityNone;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKROTATION */
-      GameEventLuaCallSetDynamicBlockRotation LuaCallSetDynamicBlockRotation;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKROTATION */
-      GameEventLuaCallSetDynamicBlockTranslation LuaCallSetDynamicBlockTranslation;
-
-      /* GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKNONE */
-      GameEventLuaCallSetDynamicBlockNone LuaCallSetDynamicBlockNone;
-
-      /* GAME_EVENT_LUA_CALL_CAMERAZOOM */
-      GameEventLuaCallCameraZoom LuaCallCameraZoom;
-
-      /* GAME_EVENT_LUA_CALL_CAMERAMOVE */
-      GameEventLuaCallCameraMove LuaCallCameraMove;
-
-    } u;		
-  };
     
   struct RecordedGameEvent {
-    float fTime;                    /* Time of event */
-    GameEvent Event;                /* Event itself */
+    MotoGameEvent *Event;           /* Event itself */
     bool bPassed;                   /* Whether we have passed it */
   };
 
-	/*===========================================================================
-	Game object
+  /*===========================================================================
+  Game object
   ===========================================================================*/
   class MotoGame {
-    public:          
-      MotoGame() {m_pLevelSrc=NULL;
-                  m_bSqueeking=false;
-                  clearStates();
-		  m_lastCallToEveryHundreath = 0.0;
-#if defined(ALLOW_GHOST)
-		  m_showGhostTimeDiff = true;
-		  m_isGhostActive = false;
-#endif
-		  m_renderer = NULL;
-      }
-      ~MotoGame() {endLevel();}     
+  public:          
+    MotoGame();
+    ~MotoGame();
     
-      /* Methods */
-      void playLevel(
+    /* update of the structure */
+    void prePlayLevel(
 #if defined(ALLOW_GHOST)    
-		     Replay *m_pGhostReplay,
+		      Replay *m_pGhostReplay,
 #endif
-		     LevelSrc *pLevelSrc, bool bIsAReplay);
-      void updateLevel(float fTimeStep,SerializedBikeState *pReplayState,Replay *p_replay);
-      void endLevel(void);
-      
-      void touchEntity(Entity *pEntity,bool bHead, bool bEnableScript); 
-      void deleteEntity(Entity *pEntity);
-      int countEntitiesByType(EntityType Type);
-      Entity *findEntity(const std::string &ID);
-      
-      void clearStates(void);
-      
-      void gameMessage(std::string Text,bool bOnce = false);
-      void clearGameMessages(void);
-      
-      void getSerializedBikeState(SerializedBikeState *pState);
-      static void unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents);
-      void interpolateGameState(SerializedBikeState *pA,SerializedBikeState *pB,SerializedBikeState *p,float t);
+		      LevelSrc *pLevelSrc,
+		      Replay *recordingReplay,
+		      bool bIsAReplay);
 
-      float getBikeEngineRPM(void);
-      float getBikeEngineSpeed();
+    void playLevel(
+#if defined(ALLOW_GHOST)    
+		   Replay *m_pGhostReplay,
+#endif
+		   LevelSrc *pLevelSrc, bool bIsAReplay);
+    void updateLevel(float fTimeStep,SerializedBikeState *pReplayState,Replay *p_replay);
+    void endLevel(void);
 
-      GameEvent *createGameEvent(GameEventType Type);
-      void destroyGameEvent(GameEvent *p_event);
-      GameEvent *getNextGameEvent(void);
-      int getNumPendingGameEvents(void);
-      void cleanEventsQueue();
+    /* entities */
+    void touchEntity(Entity *pEntity, bool bHead); 
+    void deleteEntity(Entity *pEntity);
+    int countEntitiesByType(EntityType Type);
+    Entity *findEntity(const std::string &ID);
+    Entity *getEntityByID(const std::string &ID);      
+
+    /* messages */
+    void gameMessage(std::string Text,
+		     bool bOnce = false,
+		     float fDuration = MOTOGAME_DEFAULT_GAME_MESSAGE_DURATION);
+    void clearGameMessages();
+    void updateGameMessages();
+    std::vector<GameMessage *> &getGameMessage(void) {return m_GameMessages;}
       
-      void setPlayerPosition(float x,float y,bool bFaceRight);
-      const Vector2f &getPlayerPosition(void);
-      bool getPlayerFaceDir(void);
+    /* serialization */
+    void getSerializedBikeState(SerializedBikeState *pState);
+    static void unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents);
+    void interpolateGameState(SerializedBikeState *pA,SerializedBikeState *pB,SerializedBikeState *p,float t);
 
-      Entity *getEntityByID(const std::string &ID);
+    /* events */
+    void createGameEvent(MotoGameEvent *p_event);
+    void destroyGameEvent(MotoGameEvent *p_event);
+    MotoGameEvent* getNextGameEvent();
+    int getNumPendingGameEvents();
+    void cleanEventsQueue();
+    void executeEvents(Replay *p_replay);
+
+    /* information about engine */
+    float getBikeEngineRPM(void);
+    float getBikeEngineSpeed();
       
-      /* Direct Lua interaction methods */
-      bool scriptCallBool(std::string FuncName,bool bDefault=false);
-      void scriptCallVoid(std::string FuncName);
-      void scriptCallTblVoid(std::string Table,std::string FuncName);
+    /* player */
+    void setPlayerPosition(float x,float y,bool bFaceRight);
+    const Vector2f &getPlayerPosition(void);
+    bool getPlayerFaceDir(void);
+    void setDeathAnim(bool b) {m_bDeathAnimEnabled=b;}
+    
+    /* Direct Lua interaction methods */
+    bool scriptCallBool(std::string FuncName,bool bDefault=false);
+    void scriptCallVoid(std::string FuncName);
+    void scriptCallTblVoid(std::string Table,std::string FuncName);
+    void scriptCallVoidNumberArg(std::string FuncName, int n);
 
-      /* Data interface */
-      void resetAutoDisabler(void) {m_nStillFrames=0;}
-      bool isInitOK(void) {return m_bLevelInitSuccess;}
-      bool isFinished(void) {return m_bFinished;}
-      bool isDead(void) {return m_bDead;}
-      LevelSrc *getLevelSrc(void) {return m_pLevelSrc;}
-      std::vector<ConvexBlock *> &getBlocks(void) {return m_Blocks;}
-      BikeState *getBikeState(void) {return &m_BikeS;}
-      BikeParams *getBikeParams() { return &m_BikeP;}
-      bool isSqueeking(void) {return m_bSqueeking;}
-      float howMuchSqueek(void) {return m_fHowMuchSqueek;}
+    /* Data interface */
+    void resetAutoDisabler(void) {m_nStillFrames=0;}
+    bool isInitOK(void) {return m_bLevelInitSuccess;}
+    bool isFinished(void) {return m_bFinished;}
+    bool isDead(void) {return m_bDead;}
+    LevelSrc *getLevelSrc(void) {return m_pLevelSrc;}
+    std::vector<ConvexBlock *> &getBlocks(void) {return m_Blocks;}
+    BikeState *getBikeState(void) {return &m_BikeS;}
+    BikeParams *getBikeParams() { return &m_BikeP;}
+    bool isSqueeking(void) {return m_bSqueeking;}
+    float howMuchSqueek(void) {return m_fHowMuchSqueek;}
 
 #if defined(ALLOW_GHOST)
       BikeState *getGhostBikeState(void) {return &m_GhostBikeS;}
@@ -742,8 +502,8 @@ namespace vapp {
       std::vector<float> m_ghostLastStrawberries;
       float m_myDiffOfGhost; /* time diff between the ghost and the player */
 #endif
+
       BikeController *getBikeController(void) {return &m_BikeC;}
-      std::vector<GameMessage *> &getGameMessage(void) {return m_GameMessages;}
       std::vector<Entity *> &getEntities(void) {return m_Entities;}
       std::vector<DynamicBlock *> &getDynBlocks(void) {return m_DynBlocks;}
       std::vector<OverlayEdge *> &getOverlayEdges(void) {return m_OvEdges;}
@@ -774,6 +534,7 @@ namespace vapp {
       void SetEntityPos(String pEntityID, float pX, float pY);
       void PlaceInGameArrow(float pX, float pY, float pAngle);
       void PlaceScreenArrow(float pX, float pY, float pAngle);
+      void HideArrow();
       void MoveBlock(String pBlockID, float pX, float pY);
       void SetBlockPos(String pBlockID, float pX, float pY);
       void SetBlockCenter(String pBlockID, float pX, float pY);
@@ -784,9 +545,28 @@ namespace vapp {
       void CameraZoom(float pZoom);
       void CameraMove(float p_x, float p_y);
 
-    private:         
+      void killPlayer();
+      void playerEntersZone(LevelZone *pZone);
+      void playerLeavesZone(LevelZone *pZone);
+      void playerTouchesEntity(std::string p_entityID, bool p_bTouchedWithHead);
+      void entityDestroyed(std::string p_entityID, EntityType p_type,
+			   float p_fSize, float p_fPosX, float p_fPosY);
+      void addDynamicObject(SDynamicObject* p_obj);
+      void removeSDynamicOfObject(std::string pObject);
+      void addPenalityTime(float fTime);
+
+      void revertEntityDestroyed(std::string p_entityID);
+      void createKillEntityEvent(std::string p_entityID);
+
+      unsigned int getNbRemainingStrawberries();
+      void makePlayerWin();
+
+      void setBodyDetach(bool state);
+      void stopBikeControls();
+
+  private:         
       /* Data */
-      std::queue<GameEvent*> m_GameEventQueue;
+      std::queue<MotoGameEvent*> m_GameEventQueue;
       
       float m_fTime,m_fNextAttitudeCon;
       float m_fFinishTime,m_fAttitudeCon;
@@ -798,6 +578,7 @@ namespace vapp {
       
       bool m_bSqueeking;
       float m_fHowMuchSqueek;
+      bool m_bDeathAnimEnabled;
       bool m_bLevelInitSuccess;
 
       Vector2f m_PhysGravity; /* gravity */
@@ -811,6 +592,7 @@ namespace vapp {
                                              level */
       std::vector<ConvexBlock *> m_Blocks;/* Blocks */
       std::vector<Entity *> m_Entities;   /* Entities */
+      std::vector<Entity *> m_DestroyedEntities; /* destroyed entities */
       dWorldID m_WorldID;                 /* World ID */
       
       std::vector<OverlayEdge *> m_OvEdges;/* Overlay edges */
@@ -833,6 +615,9 @@ namespace vapp {
       /* count somersault */
       SomersaultCounter m_somersaultCounter;
 
+      bool bFrontWheelTouching;
+      bool bRearWheelTouching;
+
       std::vector<SDynamicObject*> m_SDynamicObjects;
 
       BikeController m_BikeC;             /* Bike controller */
@@ -847,6 +632,7 @@ namespace vapp {
       Vector2f m_PrevFrontWheelP;         /* Prev. front wheel position */
       Vector2f m_PrevHeadP;
       Vector2f m_PrevHead2P;
+      Vector2f m_PrevActiveHead;
       
       /* Wheels spinning dirt up... muzakka! :D */
       bool m_bWheelSpin;                  /* Do it captain */
@@ -899,6 +685,8 @@ namespace vapp {
             
       dJointGroupID m_ContactGroup;       /* Contact joint group */     
       
+      bool m_bodyDetach;
+
       /* Teleport next frame? */
       bool m_bTeleport;
       GameReqPlayerPos m_TeleportDest; 
@@ -906,6 +694,10 @@ namespace vapp {
       /* for EveryHundreath function */
       float m_lastCallToEveryHundreath;
             
+      bool m_isScriptActiv; /* change this variable to activ/desactiv scripting */
+
+      void clearStates();
+
       /* Helpers */
       void _GenerateLevel(void);          /* Called by playLevel() to 
                                              prepare the level */
@@ -920,9 +712,11 @@ namespace vapp {
       void _UpdateZones(void);
       Entity *_SpawnEntity(std::string ID,EntityType Type,Vector2f Pos,LevelEntity *pSrc);
       void _KillEntity(Entity *pEnt);
+      void CleanEntities(); /* clean memories of entities */
       EntityType _TransEntityType(std::string Name);
       EdgeEffect _TransEdgeEffect(std::string Name);
       void _UpdateEntities(void);
+      bool touchEntityBodyExceptHead(const BikeState &pBike, const Entity &p_entity);
       void _UpdateGameState(SerializedBikeState *pReplayState);
       /* static */ void _UpdateStateFromReplay(SerializedBikeState *pReplayState,BikeState *pBikeS);
 
@@ -934,17 +728,15 @@ namespace vapp {
 
       void cleanScriptDynamicObjects();
       void nextStateScriptDynamicObjects();
-      void removeSDynamicOfObject(std::string pObject);
 
-      char _MapCoordTo8Bits(float fRef,float fMaxDiff,float fCoord);
-      float _Map8BitsToCoord(float fRef,float fMaxDiff,char c);
+      signed char _MapCoordTo8Bits(float fRef,float fMaxDiff,float fCoord);
+      float _Map8BitsToCoord(float fRef,float fMaxDiff,signed char c);
       unsigned short _MatrixTo16Bits(const float *pfMatrix);
       void _16BitsToMatrix(unsigned short n16,float *pfMatrix);
-      void _SerializeGameEventQueue(DBuffer &Buffer,GameEvent *pEvent);
+      void _SerializeGameEventQueue(DBuffer &Buffer,MotoGameEvent *pEvent);
       
       void _UpdateReplayEvents(Replay *p_replay);
-      void _HandleReplayEvent(GameEvent *pEvent);
-      void _HandleReverseReplayEvent(GameEvent *pEvent);
+      void _HandleReplayEvent(MotoGameEvent *pEvent);
       
       void _UpdateDynamicCollisionLines(void);
       
@@ -956,6 +748,6 @@ namespace vapp {
       void _PrepareRider(Vector2f StartPos);
     };
 
-};
+}
 
 #endif
