@@ -34,6 +34,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <iostream>
 #include <locale>
 
+#include <arch/SwapEndian.h>
+
 namespace vapp {
 
   bool Replay::m_bEnableCompression = true;
@@ -54,8 +56,10 @@ namespace vapp {
   
   void Replay::_FreeReplay(void) {
     /* Get rid of replay events */
-    for(int i=0;i<m_ReplayEvents.size();i++)
+    for(int i=0;i<m_ReplayEvents.size();i++) {
+      delete m_ReplayEvents[i]->Event;
       delete m_ReplayEvents[i];
+    }
     m_ReplayEvents.clear();
 
     /* Dealloc chunks */
@@ -96,18 +100,18 @@ namespace vapp {
     
     /* Write header */
     FS::writeByte(pfh,1); /* Version: 1 */
-    FS::writeInt(pfh,0x12345678); /* Endianness guard */
+    FS::writeInt_LE(pfh,0x12345678); /* Endianness guard */
     FS::writeString(pfh,m_LevelID);
     FS::writeString(pfh,m_PlayerName);
-    FS::writeFloat(pfh,m_fFrameRate);
-    FS::writeInt(pfh,m_nStateSize);
+    FS::writeFloat_LE(pfh,m_fFrameRate);
+    FS::writeInt_LE(pfh,m_nStateSize);
     FS::writeBool(pfh,m_bFinished);
-    FS::writeFloat(pfh,m_fFinishTime);
+    FS::writeFloat_LE(pfh,m_fFinishTime);
     
     /* Events */
     const char *pcUncompressedEvents = convertOutputToInput();
     int nUncompressedEventsSize = numRemainingBytes();
-    FS::writeInt(pfh,nUncompressedEventsSize);
+    FS::writeInt_LE(pfh,nUncompressedEventsSize);
     
     /* Compression? */
     if(m_bEnableCompression) {
@@ -124,7 +128,7 @@ namespace vapp {
       else {
         /* OK */        
         FS::writeBool(pfh,true); /* compression: true */
-        FS::writeInt(pfh,nDestLen);
+        FS::writeInt_LE(pfh,nDestLen);
         FS::writeBuf(pfh,(char *)pcCompressedEvents,nDestLen);
       }
       delete [] pcCompressedEvents;
@@ -136,13 +140,13 @@ namespace vapp {
     }
     
     /* Chunks */
-    if(m_Chunks.empty()) FS::writeInt(pfh,0);
+    if(m_Chunks.empty()) FS::writeInt_LE(pfh,0);
     else {        
-      FS::writeInt(pfh,m_Chunks.size());
+      FS::writeInt_LE(pfh,m_Chunks.size());
     
       /* Write chunks */    
       for(int i=0;i<m_Chunks.size();i++) {
-        FS::writeInt(pfh,m_Chunks[i].nNumStates);
+        FS::writeInt_LE(pfh,m_Chunks[i].nNumStates);
 
         /* Compression enabled? */
         if(m_bEnableCompression) {
@@ -159,7 +163,7 @@ namespace vapp {
           else {
             /* Compressed ok */
             FS::writeBool(pfh,true); /* compression: true */
-            FS::writeInt(pfh,nDestLen);
+            FS::writeInt_LE(pfh,nDestLen);
             FS::writeBuf(pfh,(char *)pcCompressed,nDestLen);
           }
           delete [] pcCompressed;        
@@ -176,16 +180,15 @@ namespace vapp {
   }
   
   std::string Replay::openReplay(const std::string &FileName,float *pfFrameRate,std::string &Player) {
-
-    /* Open file for input */
-    FileHandle *pfh = FS::openIFile(std::string("Replays/") + FileName);
-    if(pfh == NULL) {
-      /* Try adding a .rpl extension */
-      pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
-      if(pfh == NULL) {    
-        /* Finally try opening as if it is a full path */
-        pfh = FS::openIFile(FileName);
-        if(pfh == NULL) {
+    /* Try opening as if it is a full path */
+    FileHandle *pfh = FS::openIFile(FileName);
+    if(pfh == NULL) {        
+      /* Open file for input */
+      pfh = FS::openIFile(std::string("Replays/") + FileName);
+      if(pfh == NULL) {
+        /* Try adding a .rpl extension */
+        pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
+        if(pfh == NULL) {    
           Log("** Warning ** : Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
           return "";
         }
@@ -203,7 +206,7 @@ namespace vapp {
     }
     else {
       /* Little/big endian safety check */
-      if(FS::readInt(pfh) != 0x12345678) {
+      if(FS::readInt_LE(pfh) != 0x12345678) {
         FS::closeFile(pfh);
         Log("** Warning ** : Sorry, the replay you're trying to open are not endian-compatible with your computer!");
         return "";        
@@ -216,26 +219,26 @@ namespace vapp {
       Player = m_PlayerName = FS::readString(pfh);
       
       /* Read replay frame rate */
-      m_fFrameRate = FS::readFloat(pfh);
+      m_fFrameRate = FS::readFloat_LE(pfh);
       if(pfFrameRate != NULL) *pfFrameRate = m_fFrameRate;            
 
       /* Read state size */
-      m_nStateSize = FS::readInt(pfh);
+      m_nStateSize = FS::readInt_LE(pfh);
       
       /* Read finish time if any */
       m_bFinished = FS::readBool(pfh);
-      m_fFinishTime = FS::readFloat(pfh);
+      m_fFinishTime = FS::readFloat_LE(pfh);
 
       /* Version 1 includes event data */
       if(nVersion == 1) {
         /* Read uncompressed size */
-        m_nInputEventsDataSize = FS::readInt(pfh);
+        m_nInputEventsDataSize = FS::readInt_LE(pfh);
         m_pcInputEventsData = new char [m_nInputEventsDataSize];
         
         /* Compressed? */
         if(FS::readBool(pfh)) {
           /* Compressed */          
-          int nCompressedEventsSize = FS::readInt(pfh);
+          int nCompressedEventsSize = FS::readInt_LE(pfh);
           
           char *pcCompressedEvents = new char [nCompressedEventsSize];
           FS::readBuf(pfh,pcCompressedEvents,nCompressedEventsSize);
@@ -266,16 +269,16 @@ namespace vapp {
       }
       
       /* Read chunks */
-      int nNumChunks = FS::readInt(pfh);
+      int nNumChunks = FS::readInt_LE(pfh);
       for(int i=0;i<nNumChunks;i++) {        
         ReplayStateChunk Chunk;        
-        Chunk.nNumStates = FS::readInt(pfh);
+        Chunk.nNumStates = FS::readInt_LE(pfh);
         Chunk.pcChunkData = new char [Chunk.nNumStates * m_nStateSize];
         
         /* Compressed or not compressed? */
         if(FS::readBool(pfh)) {
           /* Compressed! - read compressed size */
-          int nCompressedSize = FS::readInt(pfh);
+          int nCompressedSize = FS::readInt_LE(pfh);
           
           /* Read compressed data */
           unsigned char *pcCompressed = new unsigned char [nCompressedSize];
@@ -319,28 +322,33 @@ namespace vapp {
     return m_LevelID;
   }
       
-  void Replay::storeState(const char *pcState) {
+  void Replay::storeState(const SerializedBikeState& state) {
+    char *addr;
+    
     if(m_Chunks.empty()) {
       ReplayStateChunk Chunk;
       Chunk.pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
       Chunk.nNumStates = 1;
-      memcpy(Chunk.pcChunkData,pcState,m_nStateSize);
+      addr = Chunk.pcChunkData;
       m_Chunks.push_back(Chunk);
     }
     else {
       int i = m_Chunks.size() - 1;
       if(m_Chunks[i].nNumStates < STATES_PER_CHUNK) {
-        memcpy(&m_Chunks[i].pcChunkData[m_Chunks[i].nNumStates * m_nStateSize],pcState,m_nStateSize);
+        addr = &m_Chunks[i].pcChunkData[m_Chunks[i].nNumStates * m_nStateSize];
         m_Chunks[i].nNumStates++;
       }
       else {
         ReplayStateChunk Chunk;
         Chunk.pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
         Chunk.nNumStates = 1;
-        memcpy(Chunk.pcChunkData,pcState,m_nStateSize);
+        addr = Chunk.pcChunkData;
         m_Chunks.push_back(Chunk);        
       }
     }
+    
+    memcpy(addr,(const char*)&state,m_nStateSize);
+	SwapEndian::LittleSerializedBikeState(*(SerializedBikeState*)addr);
   }
   
   bool Replay::nextNormalState() {
@@ -348,13 +356,11 @@ namespace vapp {
   }
 
   bool Replay::nextState(float p_frames) {
+    m_bEndOfFile = false;
     if(m_is_paused) {
       return ! m_bEndOfFile;
     }
 
-    /* put endoffile to false ; and in the following line, put it to true if required*/
-    m_bEndOfFile = false;
-
     m_nCurState += p_frames;
 
     /* end or start of a chunk */
@@ -362,15 +368,15 @@ namespace vapp {
 
       /* go on the following chunk */
       if(m_nCurState >= STATES_PER_CHUNK) { /* end of a chunk */
-	if(m_nCurChunk < m_Chunks.size() -1) {	
+	if(m_nCurChunk < m_Chunks.size() -1) {
+	  m_nCurState = m_nCurState - m_Chunks[m_nCurChunk].nNumStates -1;
 	  m_nCurChunk++;
-	  m_nCurState = m_nCurState - m_Chunks[m_nCurChunk-1].nNumStates -1;
 	}
       } else {
 	if(m_nCurState < 0.0) { /* start of a chunk */
-	  if(m_nCurChunk > 0) {	
+	  if(m_nCurChunk > 0) {
+	    m_nCurState = m_nCurState + m_Chunks[m_nCurChunk-1].nNumStates -1;
 	    m_nCurChunk--;
-	    m_nCurState = m_Chunks[m_nCurChunk-1].nNumStates -1 + m_nCurState;
 	  }
 	}
       }
@@ -382,7 +388,6 @@ namespace vapp {
 
       /* if that's end */
       if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
-	m_bEndOfFile = true;
 	m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
 	return false;
       }
@@ -393,7 +398,6 @@ namespace vapp {
 
       /* if that's end */
       if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
-	m_bEndOfFile = true;
 	m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
 	return false;
       }
@@ -402,18 +406,25 @@ namespace vapp {
     return true;
   }
 
-  void Replay::loadState(char *pcState) {
+  void Replay::loadState(SerializedBikeState& state) {
     /* (11th july, 2006) rasmus: i've swapped the two following lines, it apparently fixes
        interpolation in replays, but i don't know if it break something else */  
-    peekState(pcState);
-    if(nextNormalState()) { /* do nothing */ }
+    peekState(state);
+
+    m_bEndOfFile = (m_nCurChunk       == m_Chunks.size()-1 && 
+	 	    (int)m_nCurState  == m_Chunks[m_nCurChunk].nNumStates-1);
+
+    if(m_bEndOfFile == false) {
+      if(nextNormalState()) { /* do nothing */ }
+    }
   }
   
-  void Replay::peekState(char *pcState) {
+  void Replay::peekState(SerializedBikeState& state) {
     /* Like loadState() but this one does not advance the cursor... it just takes a peek */
-    memcpy(pcState,
+    memcpy((char *)&state,
 	   &m_Chunks[m_nCurChunk].pcChunkData[((int)m_nCurState)*m_nStateSize],
 	   m_nStateSize);
+	  SwapEndian::LittleSerializedBikeState(state);
   }
 
   //std::vector<ReplayInfo *> Replay::createReplayList(const std::string &PlayerName,const std::string &LevelIDCheck) {
@@ -429,13 +440,13 @@ namespace vapp {
     //  if(pfh != NULL) {
     //    int nVersion = FS::readByte(pfh);
     //    if(nVersion == 0 || nVersion == 1) {
-    //      if(FS::readInt(pfh) == 0x12345678) {                  
+    //      if(FS::readInt_LE(pfh) == 0x12345678) {                  
     //        std::string LevelID = FS::readString(pfh);
     //        std::string Player = FS::readString(pfh);
-    //        float fFrameRate = FS::readFloat(pfh);
-    //        int nStateSize = FS::readInt(pfh);
+    //        float fFrameRate = FS::readFloat_LE(pfh);
+    //        int nStateSize = FS::readInt_LE(pfh);
     //        bool bFinished = FS::readBool(pfh);
-    //        float fFinishTime = FS::readFloat(pfh);
+    //        float fFinishTime = FS::readFloat_LE(pfh);
     //        
     //        if((PlayerName=="" || PlayerName==Player) && FS::getFileBaseName(ReplayFiles[i]) != "Latest") {
     //          if(LevelIDCheck=="" || LevelID==LevelIDCheck) {
@@ -558,17 +569,17 @@ namespace vapp {
 	return NULL;
       }
 
-      if(FS::readInt(pfh) != 0x12345678) {   
+      if(FS::readInt_LE(pfh) != 0x12345678) {   
 	FS::closeFile(pfh);
 	return NULL;
       }
   
       std::string LevelID = FS::readString(pfh);
       std::string Player  = FS::readString(pfh);
-      float fFrameRate    = FS::readFloat(pfh);
-      int nStateSize      = FS::readInt(pfh);
+      float fFrameRate    = FS::readFloat_LE(pfh);
+      int nStateSize      = FS::readInt_LE(pfh);
       bool bFinished      = FS::readBool(pfh);
-      float fFinishTime   = FS::readFloat(pfh);
+      float fFinishTime   = FS::readFloat_LE(pfh);
                 
       ReplayInfo *pRpl     = new ReplayInfo;
       int nTimeStamp       = FS::getFileTimeStamp("Replays/" + p_ReplayName + ".rpl");
@@ -590,4 +601,4 @@ namespace vapp {
       return pRpl;
   }
 
-};
+}
