@@ -85,9 +85,11 @@ namespace vapp {
       virtual ~Replay();
       
       /* Methods */
-      void storeState(const char *pcState);
-      void loadState(char *pcState); /* go and get the next state */  
-      void peekState(char *pcState); /* get current state */
+      void storeState(const SerializedBikeState& state);
+      /* go and get the next state */
+      void loadState(SerializedBikeState& state);  
+      void peekState(SerializedBikeState& state); /* get current state */
+      
       void createReplay(const std::string &FileName,const std::string &LevelID,const std::string &Player,float fFrameRate,int nStateSize);
       void saveReplay(void);
       std::string openReplay(const std::string &FileName,float *pfFrameRate,std::string &Player);
@@ -102,10 +104,14 @@ namespace vapp {
       void faster();
       void slower();
       float getSpeed() const; /* get multiple factor of the replay */
+      
+      void setSpeed(float f) {m_speed_factor = f;}
+      bool isPaused(void) {return m_is_paused;}
 
       /* Data interface */
       bool didFinish(void) {return m_bFinished;}
-      float getFinishTime(void) {return m_fFinishTime;}      
+      float getFinishTime(void) {return m_fFinishTime;}     
+      float getFrameRate(void) {return m_fFrameRate;} 
       const std::string &getPlayerName(void) {return m_PlayerName;}
       bool endOfFile(void) {return m_bEndOfFile;}
       
@@ -149,7 +155,7 @@ namespace vapp {
       std::vector<RecordedGameEvent *> m_ReplayEvents;
   };
 
-};
+}
 
 #endif
 
