@@ -30,30 +30,18 @@ class SomersaultCounter {
   ~SomersaultCounter();
 
   void init();
-  bool update(vapp::Vector2f p_wheel1, vapp::Vector2f p_wheel2);
+  bool update(double p_angle, bool &bCounterclock);
 
   int getTotalClockwise();
   int getTotalCounterClockwise();
   int getTotal();
 
   private:
-  int getCurrentState(vapp::Vector2f p_wheel1, vapp::Vector2f p_wheel2);
-
-  /* five states : the five last one */
-  /* five because we must come back to the original one */
-  /*
-       |
-    4  |  3
-  -----O------
-    1  |  2
-       |
-
-  */
   bool m_last_initialized;
-  int m_last_state;
-  int m_min_move;
-  int m_max_move;
-  int m_current_move;
+  double m_last_state;
+  double m_min_move;
+  double m_max_move;
+  double m_current_move;
 
   int m_nbClockwise;
   int m_nbCounterClockwise;
