@@ -123,8 +123,8 @@ Theme::~Theme() {
   m_texMan.unloadTextures();
 }
  
-vapp::Texture* Theme::loadTexture(std::string p_fileName, bool bSmall, bool bClamp, bool bFilter) {
-  return m_texMan.loadTexture(p_fileName.c_str(), bSmall, bClamp, bFilter);
+vapp::Texture* Theme::loadTexture(std::string p_fileName, bool bSmall, bool bClamp, vapp::FilterMode eFilterMode) {
+  return m_texMan.loadTexture(p_fileName.c_str(), bSmall, bClamp, eFilterMode);
 }
 
 std::vector<std::string>* Theme::getRequiredFiles() {
@@ -203,6 +203,8 @@ void Theme::loadSpritesFromXML(TiXmlElement *p_ThemeXmlDataElement) {
       newTextureSpriteFromXML(pVarElem);
     } else if(v_spriteType == "UI") {
       newUISpriteFromXML(pVarElem);
+    } else if(v_spriteType == "EdgeEffect") {
+      newEdgeEffectSpriteFromXML(pVarElem);
     } else {
       vapp::Log("Warning: unknown type '%s' in theme file !", v_spriteType.c_str());
     }
@@ -389,6 +391,35 @@ void Theme::newEffectSpriteFromXML(TiXmlElement *pVarElem) {
   m_requiredFiles.push_back(THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
 }
 
+void Theme::newEdgeEffectSpriteFromXML(TiXmlElement *pVarElem) {
+  std::string v_name;
+  std::string v_fileName;
+  std::string v_scale;
+  std::string v_depth;
+  const char *pc;
+
+  pc = pVarElem->Attribute("name");
+  if(pc == NULL) {return;}
+  v_name = pc;
+
+  pc = pVarElem->Attribute("file");
+  if(pc == NULL) {return;}
+  v_fileName = pc;
+
+  pc = pVarElem->Attribute("scale");
+  if(pc == NULL) {return;}
+  v_scale = pc;
+
+  pc = pVarElem->Attribute("depth");
+  if(pc == NULL) {return;}
+  v_depth = pc;
+
+  m_sprites.push_back(new EdgeEffectSprite(this, v_name, v_fileName,
+					   atof(v_scale.c_str()),
+					   atof(v_depth.c_str())));
+  m_requiredFiles.push_back(THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+}
+
 void Theme::newFontSpriteFromXML(TiXmlElement *pVarElem) {
   std::string v_name;
   std::string v_fileName;
@@ -475,7 +506,7 @@ Sprite::Sprite(Theme* p_associated_theme, std::string v_name) {
 Sprite::~Sprite() {
 }
 
-vapp::Texture* Sprite::getTexture(bool bSmall, bool bClamp, bool bFilter) {
+vapp::Texture* Sprite::getTexture(bool bSmall, bool bClamp, vapp::FilterMode eFilterMode) {
   vapp::Texture* v_currentTexture;
 
   v_currentTexture = getCurrentTexture();
@@ -483,7 +514,7 @@ vapp::Texture* Sprite::getTexture(bool bSmall, bool bClamp, bool bFilter) {
     v_currentTexture = m_associated_theme->loadTexture(getCurrentTextureFileName(),
 						       bSmall,
 						       bClamp,
-						       bFilter);
+						       eFilterMode);
     if(v_currentTexture == NULL) { 
       throw vapp::Exception("Unable to load texture '" + getCurrentTextureFileName() + "'");
     }
@@ -695,6 +726,30 @@ std::string EffectSprite::getFileDir() {
   return THEME_EFFECT_SPRITE_FILE_DIR;
 }
 
+EdgeEffectSprite::EdgeEffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename, float p_fScale, float p_fDepth) : SimpleFrameSprite(p_associated_theme, p_name, p_filename) {
+  m_fScale = p_fScale;
+  m_fDepth = p_fDepth;
+}
+
+EdgeEffectSprite::~EdgeEffectSprite() {
+}
+
+enum SpriteType EdgeEffectSprite::getType() {
+  return SPRITE_TYPE_EDGEEFFECT;
+}
+
+std::string EdgeEffectSprite::getFileDir() {
+  return THEME_EDGEEFFECT_SPRITE_FILE_DIR;
+}
+
+float EdgeEffectSprite::getScale() const {
+  return m_fScale;
+}
+
+float EdgeEffectSprite::getDepth() const {
+  return m_fDepth;
+}
+
 FontSprite::FontSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
 }
 
@@ -857,9 +912,9 @@ bool ThemeChoicer::ExistThemeName(std::string p_themeName) {
 
 #if defined(SUPPORT_WEBACCESS)
   ThemeChoicer::ThemeChoicer(vapp::WWWAppInterface *p_WebApp,
-			     const ProxySettings *p_proxy_settings) {
+			     const ProxySettings *p_proxy_settings) {			     			     
 #else
-  ThemeChoicer::ThemeChoicer() {  
+  ThemeChoicer::ThemeChoicer(void) {  
 #endif
 
 #if defined(SUPPORT_WEBACCESS)
@@ -944,7 +999,8 @@ void ThemeChoicer::initList() {
 	if(ExistThemeName(v_name) == false) {
 	  m_choices.push_back(new ThemeChoice(v_name, v_themesFiles[i], true));
 	} else {
-	  vapp::Log(std::string("Theme " + v_name + " is present several times").c_str());
+	  // no warning for non user dir
+	  // vapp::Log(std::string("Theme " + v_name + " is present several times").c_str());
 	}
       }
     } catch(vapp::Exception &e) {
@@ -1027,6 +1083,10 @@ void ThemeChoicer::updateFromWWW() {
   initList();
 }
 
+bool ThemeChoicer::isUpdatableThemeFromWWW(ThemeChoice* pThemeChoice) {
+  return m_webThemes->isUpgradable(pThemeChoice);
+}
+
 void ThemeChoicer::updateThemeFromWWW(ThemeChoice* pThemeChoice) {
   m_webThemes->upgrade(pThemeChoice);
 }
