@@ -33,6 +33,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VFileIO.h"
 #include "arch/SwapEndian.h"
 
+#ifdef USE_GETTEXT
+#include "Locales.h"
+#endif
+
 namespace vapp {
 
   /*===========================================================================
@@ -43,17 +47,20 @@ namespace vapp {
   
   void LogRaw(const char *pcMsg) {
     if(!g_bQuietLog) {
-      #if defined(_MSC_VER) && defined(_DEBUG)
-        printf("%s\n",pcMsg); /* I like my stdout when debugging :P */
-      #else
-        if(g_bVerbose) 
-          printf("%s\n",pcMsg); /* also write to stdout */
-      #endif
-
+      Verbose(pcMsg);
       FS::writeLog(pcMsg);
     }
   }
   
+  void Verbose(const char *pcMsg) {
+#if defined(_MSC_VER) && defined(_DEBUG)
+    printf("%s\n",pcMsg); /* I like my stdout when debugging :P */
+#else
+    if(g_bVerbose) 
+    printf("%s\n",pcMsg); /* also write to stdout */
+#endif
+  }
+
   void Log(const char *pcFmt,...) {
     va_list List;
     char cBuf[1024];
@@ -131,6 +138,9 @@ namespace vapp {
   void App::run(int nNumArgs,char **ppcArgs) {
 
     /* init endian system */ 
+#ifdef USE_GETTEXT
+    Locales::init();
+#endif
     SwapEndian::Swap_Init();
     //if(SwapEndian::bigendien) {
     //  Log("systeme bigendien");
@@ -223,7 +233,7 @@ namespace vapp {
         }
           
         /* Clear screen */  
-        glClear(GL_COLOR_BUFFER_BIT);
+        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
       }
       
       /* Update user app */
@@ -335,8 +345,7 @@ namespace vapp {
 		  /* In windowed mode we can't tinker with the bit-depth */
 		  m_nDispBPP=pVidInfo->vfmt->BitsPerPixel; 			
 
-	  /* Setup GL stuff - note we setup depth too, even though it isn't used
-	    at all. This is simply for completeness :) */
+	  /* Setup GL stuff */
 	  /* 2005-10-05 ... note that we no longer ask for explicit settings... it's
 	                    better to do it per auto */
 	  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);	
@@ -351,8 +360,14 @@ namespace vapp {
 	        "                Now SDL will try determining a proper mode itself.",m_nDispWidth,m_nDispHeight,m_nDispBPP);
 	  
 	    /* Hmm, try letting it decide the BPP automatically */
-	    if(SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,0,nFlags)==NULL) {
-        throw Exception("SDL_SetVideoMode : " + std::string(SDL_GetError()));
+	    if(SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,0,nFlags)==NULL) {	      
+	      /* Still no luck */
+	      Log("** Warning ** : Still no luck, now we'll try 800x600 in a window.");
+	      m_nDispWidth = 800; m_nDispHeight = 600;	      
+	      m_bWindowed = true;
+  	    if(SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,0,SDL_OPENGL)==NULL) {	      
+          throw Exception("SDL_SetVideoMode : " + std::string(SDL_GetError()));
+  	    }	      
       }
     }
 		
@@ -362,6 +377,12 @@ namespace vapp {
       throw Exception("(2) SDL_GetVideoInfo : " + std::string(SDL_GetError()));
   									
 	  m_nDispBPP=pVidInfo->vfmt->BitsPerPixel;
+
+    /* Did we get a z-buffer? */    	  
+	  int nDepthBits;
+	  SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE,&nDepthBits);
+	  if(nDepthBits == 0)
+	    throw Exception("no depth buffer");  
 	
 	  /* Set window title */
 	  SDL_WM_SetCaption(m_AppName.c_str(),m_AppName.c_str());
@@ -372,6 +393,9 @@ namespace vapp {
 	  glLoadIdentity();
 	  glOrtho(0,m_nDispWidth,0,m_nDispHeight,-1,1);
 	  
+	  glClearDepth(1);
+	  glDepthFunc(GL_LEQUAL);
+	  
 	  glMatrixMode(GL_MODELVIEW);
 	  glLoadIdentity();		
 		
@@ -723,7 +747,10 @@ namespace vapp {
       } 
       else if(!strcmp(ppcArgs[i],"-noexts")) {
         m_bDontUseGLExtensions = true;
-      }     
+      }
+      else if(!strcmp(ppcArgs[i],"-nowww")) {
+        m_bNoWWW = true;
+      }    
       else if(!strcmp(ppcArgs[i],"-h") || !strcmp(ppcArgs[i],"-?") ||
               !strcmp(ppcArgs[i],"--help") || !strcmp(ppcArgs[i],"-help")) {
         printf("%s (Version %s)\n",m_AppName.c_str(),getVersionString().c_str());
@@ -740,6 +767,7 @@ namespace vapp {
         printf("\t-q\n\t\tDon't print messages to screen, and don't save them in the log.\n");
         printf("\t-v\n\t\tBe verbose.\n");
         printf("\t-noexts\n\t\tDon't use any OpenGL extensions.\n");
+        printf("\t-nowww\n\t\tDon't allow xmoto to connect on the web.\n");
         helpUserArgs();
         printf("\n");
         
@@ -756,5 +784,5 @@ namespace vapp {
       parseUserArgs(UserArgs);
   }
 
-};
+}
 
