@@ -596,7 +596,7 @@ namespace vapp {
   }   
   
   int FS::readByte(FileHandle *pfh) {
-    char v;
+    signed char v;
     if(!readBuf(pfh,(char *)&v,sizeof(v))) _ThrowFileError(pfh,"readByte -> failed");
     return v;
   } 
@@ -636,17 +636,16 @@ namespace vapp {
   
   std::string FS::readLongString(FileHandle *pfh) {
     unsigned short v;
-    v=readShort(pfh);
+    v=readShort_LE(pfh);
     char cBuf[65536];  
     if(!readBuf(pfh,(char *)cBuf,v)) _ThrowFileError(pfh,"readLongString -> failed");
     cBuf[v] = '\0';
     return std::string(cBuf);  
   }
   
+  // We use one-byte bools
   bool FS::readBool(FileHandle *pfh) {
-    bool v;
-    if(!readBuf(pfh,(char *)&v,sizeof(v))) _ThrowFileError(pfh,"readBool -> failed");
-    return v;
+    return static_cast<bool>(readByte(pfh));
   }   
 
   void FS::writeByte(FileHandle *pfh,unsigned char v) {
@@ -675,12 +674,12 @@ namespace vapp {
   }
   
   void FS::writeLongString(FileHandle *pfh,std::string v) {
-    writeShort(pfh,v.length());
+    writeShort_LE(pfh,v.length());
     if(!writeBuf(pfh,(char *)v.c_str(),v.length())) _ThrowFileError(pfh,"writeLongString -> failed");
   }
   
   void FS::writeBool(FileHandle *pfh,bool v) {
-    if(!writeBuf(pfh,(char *)&v,sizeof(v))) _ThrowFileError(pfh,"writeBool -> failed");
+    writeByte(pfh,static_cast<unsigned char>(v));
   }
   
   void FS::writeLine(FileHandle *pfh,std::string Line) {
@@ -828,22 +827,39 @@ namespace vapp {
   
   /*===========================================================================
   Delete file
-  ===========================================================================*/
-  void FS::deleteFile(const std::string &File) {
-    /* We can only delete user files */
+  ===========================================================================*/  
+  bool FS::deleteFile(const std::string &File) {
+    std::string FullFile;
+    
     if(m_UserDir == "") {
       Log("** Warning ** : No user directory, can't delete file: %s",File.c_str());
-      return;
+      return false;
     }
     
-    std::string FullFile = m_UserDir + std::string("/") + File;
-    remove(FullFile.c_str());
+    /* Absolute file path? */
+    if(isPathAbsolute(File)) {
+      /* Yeah, check if file is in user directory - otherwise we won't delete 
+         it */
+      if(File.length() < m_UserDir.length() || File.substr(0,m_UserDir.length()) != m_UserDir) return false;
+      
+      FullFile = File;
+    }
+    else {  
+      /* We can only delete user files */      
+      FullFile = m_UserDir + std::string("/") + File;
+    }
+
+    if(remove(FullFile.c_str())) {
+      return false;
+    }
+
+    return true;
   }
   
   /*===========================================================================
   Copy file
   ===========================================================================*/
-  bool FS::copyFile(const std::string &From,const std::string &To) {
+  bool FS::copyFile(const std::string &From,const std::string &To, std::string &To_really_done) {
     /* All file copying must happen inside the user directory... */
     if(m_UserDir == "") {
       Log("** Warning ** : No user directory, can't copy file '%s' to '%s'",From.c_str(),To.c_str());
@@ -894,6 +910,8 @@ namespace vapp {
     if(in != NULL) {
       FILE *out = fopen(FullTo.c_str(),"wb");
       if(out != NULL) {
+	To_really_done = FullTo;
+
         /* Get input file size */
         fseek(in,0,SEEK_END);
         int nFileSize = ftell(in);
@@ -1141,46 +1159,48 @@ namespace vapp {
   Endian-safe I/O helpers
   ===========================================================================*/
   void FS::writeShort_LE(FileHandle *pfh,short v) {    
-    writeByte(pfh,v&0xff);
-    writeByte(pfh,(v&0xff00)>>8);
+    writeShort(pfh, SwapEndian::LittleShort(v));
   }
   
   void FS::writeInt_LE(FileHandle *pfh,int v) {
-    writeByte(pfh,v&0xff);
-    writeByte(pfh,(v&0xff00)>>8);
-    writeByte(pfh,(v&0xff0000)>>16);
-    writeByte(pfh,(v&0xff000000)>>24);
+    writeInt(pfh, SwapEndian::LittleLong(v));
   }
   
   void FS::writeFloat_LE(FileHandle *pfh,float v) {
-    writeInt_LE(pfh,*((int *)&v));    
-    //unsigned char *pc=(unsigned char *)&v;
-    //for(int i=0;i<4;i++)
-    //  writeByte(pfh,pc[i]);
+    writeFloat(pfh, SwapEndian::LittleFloat(v));
   }
   
   int FS::readShort_LE(FileHandle *pfh) {
-    unsigned char c[2];
-    readBuf(pfh,(char *)c,2);
-    return c[0]|(c[1]<<8);
+    short v = readShort(pfh);
+    return SwapEndian::LittleShort(v);
   }
   
   int FS::readInt_LE(FileHandle *pfh) {
-    unsigned char c[4];
-    readBuf(pfh,(char *)c,4);
-    return c[0]|(c[1]<<8)|(c[2]<<16)|(c[3]<<24);
+    int v = readInt(pfh);
+    return SwapEndian::LittleLong(v);
   }
 
   float FS::readFloat_LE(FileHandle *pfh) {
-    float res;
-
-    unsigned char *pc=(unsigned char *)&res;
-    for(int i=0;i<4;i++)
-    pc[i]=readByte(pfh);
-
-    return res;
+    float v = readFloat(pfh);
+    return SwapEndian::LittleFloat(v);
   }  
- 
+  
+  int FS::readShort_MaybeLE(FileHandle *pfh, bool big) {
+    short v = readShort(pfh);
+    return big ? SwapEndian::BigShort(v) : SwapEndian::LittleShort(v);
+  }
+  
+  int FS::readInt_MaybeLE(FileHandle *pfh, bool big) {
+    int v = readInt(pfh);
+    return big ? SwapEndian::BigLong(v) : SwapEndian::LittleLong(v);
+  }
+  
+  float FS::readFloat_MaybeLE(FileHandle *pfh, bool big) {
+    float v = readFloat(pfh);
+    return big ? SwapEndian::BigFloat(v) : SwapEndian::LittleFloat(v);
+  }
+  
+  
   bool FS::isInUserDir(std::string p_filepath) {
     std::string v_userDir;
     std::string v_fileDir;
@@ -1226,5 +1246,5 @@ namespace vapp {
     }
   }
 
-};
+}
 
