@@ -49,7 +49,12 @@ namespace vapp {
       TiXmlDocument *m_pXML;
   };
 
-};
+
+  class XML {
+  public:
+    static std::string str2xmlstr(std::string str);
+  };
+}
 
 #endif
 
