@@ -34,6 +34,16 @@
 #include "compression/FileCompression.h"
 #include "md5sum/md5file.h"
 
+struct f_curl_download_data {
+  vapp::WWWAppInterface *v_WebApp;
+  int v_nb_files_to_download;
+  int v_nb_files_performed;
+};
+
+struct f_curl_upload_data {
+  vapp::WWWAppInterface *v_WebApp;
+};
+
 ProxySettings::ProxySettings() {
   m_server       = "";
   m_port         = -1;
@@ -271,7 +281,7 @@ void WebRoom::fillHash() {
   v_webHSXmlData = v_webHSXml.getLowLevelAccess();
 
   if(v_webHSXmlData == NULL) {
-    throw vapp::Exception("error : unable analyse xml highscore file");
+    throw vapp::Exception("error : unable to analyze xml highscore file");
   }
 
   v_webHSXmlDataElement = v_webHSXmlData->FirstChildElement("xmoto_worldrecords");
@@ -334,11 +344,11 @@ size_t FSWeb::writeData(void *ptr, size_t size, size_t nmemb, FILE *stream) {
 
 void FSWeb::downloadFileBz2(const std::string &p_local_file,
 			    const std::string &p_web_file,
-			    int (*curl_progress_callback)(void *clientp,
-							  double dltotal,
-							  double dlnow,
-							  double ultotal,
-							  double ulnow),
+			    int (*curl_progress_callback_download)(void *clientp,
+								   double dltotal,
+								   double dlnow,
+								   double ultotal,
+								   double ulnow),
 			    void *p_data,
 			    const ProxySettings *p_proxy_settings) {
   std::string v_bzFile = p_local_file + ".bz2";
@@ -347,7 +357,7 @@ void FSWeb::downloadFileBz2(const std::string &p_local_file,
   remove(v_bzFile.c_str());
   downloadFile(v_bzFile,
 	       p_web_file + ".bz2",
-	       curl_progress_callback,
+	       curl_progress_callback_download,
 	       p_data,
 	       p_proxy_settings
 	       );
@@ -363,11 +373,11 @@ void FSWeb::downloadFileBz2(const std::string &p_local_file,
 
 void FSWeb::downloadFileBz2UsingMd5(const std::string &p_local_file,
 				    const std::string &p_web_file,
-				    int (*curl_progress_callback)(void *clientp,
-								  double dltotal,
-								  double dlnow,
-								  double ultotal,
-								  double ulnow),
+				    int (*curl_progress_callback_download)(void *clientp,
+									   double dltotal,
+									   double dlnow,
+									   double ultotal,
+									   double ulnow),
 				    void *p_data,
 				    const ProxySettings *p_proxy_settings) {
   bool require_dwd = true;
@@ -401,7 +411,7 @@ void FSWeb::downloadFileBz2UsingMd5(const std::string &p_local_file,
   if(require_dwd) {
     FSWeb::downloadFileBz2(p_local_file,
 			   p_web_file,
-			   curl_progress_callback,
+			   curl_progress_callback_download,
 			   p_data,
 			   p_proxy_settings);
   }
@@ -409,15 +419,22 @@ void FSWeb::downloadFileBz2UsingMd5(const std::string &p_local_file,
 
 void FSWeb::downloadFile(const std::string &p_local_file,
 			 const std::string &p_web_file,
-			 int (*curl_progress_callback)(void *clientp,
-						       double dltotal,
-						       double dlnow,
-						       double ultotal,
-						       double ulnow),
+			 int (*curl_progress_callback_download)(void *clientp,
+								double dltotal,
+								double dlnow,
+								double ultotal,
+								double ulnow),
 			 void *p_data,
 			 const ProxySettings *p_proxy_settings
 			 ) {
 
+  vapp::Verbose(std::string("downloading " + 
+			    p_web_file     +
+			    " to "         +
+			    p_local_file
+			    ).c_str()
+		);
+
   CURL *v_curl;
   CURLcode res;
   FILE *v_destinationFile;
@@ -459,10 +476,11 @@ void FSWeb::downloadFile(const std::string &p_local_file,
     v_proxy_server = p_proxy_settings->getServer();
     v_proxy_auth_str =
       p_proxy_settings->getAuthentificationUser()
-      + " "
+      + ":"
       + p_proxy_settings->getAuthentificationPassword();
 
     if(p_proxy_settings->useDefaultServer() == false) {
+      vapp::Verbose(std::string("set proxy ->" + v_proxy_server).c_str());
       curl_easy_setopt(v_curl, CURLOPT_PROXY, v_proxy_server.c_str());
     }
     
@@ -473,15 +491,16 @@ void FSWeb::downloadFile(const std::string &p_local_file,
     curl_easy_setopt(v_curl, CURLOPT_PROXYTYPE, p_proxy_settings->getType());
    
     if(p_proxy_settings->useDefaultAuthentification() == false) {
+      vapp::Verbose(std::string("set proxy authentification ->" + v_proxy_auth_str).c_str());
       curl_easy_setopt(v_curl, CURLOPT_PROXYUSERPWD,
 		       v_proxy_auth_str.c_str());
     }
   }
   /* ***** */
 
-  if(curl_progress_callback != NULL) {
+  if(curl_progress_callback_download != NULL) {
     curl_easy_setopt(v_curl, CURLOPT_NOPROGRESS, false);
-    curl_easy_setopt(v_curl, CURLOPT_PROGRESSFUNCTION, curl_progress_callback);
+    curl_easy_setopt(v_curl, CURLOPT_PROGRESSFUNCTION, curl_progress_callback_download);
     curl_easy_setopt(v_curl, CURLOPT_PROGRESSDATA, p_data);
   }
 
@@ -512,6 +531,187 @@ void FSWeb::downloadFile(const std::string &p_local_file,
   }
 }
 
+void FSWeb::uploadReplay(std::string p_replayFilename,
+			 std::string p_id_room,
+			 std::string p_login,
+			 std::string p_password,
+			 std::string p_url_to_transfert,
+			 vapp::WWWAppInterface *p_WebApp,
+			 const ProxySettings *p_proxy_settings,
+			 bool &p_msg_status,
+			 std::string &p_msg) {
+  CURL *v_curl;
+  CURLcode v_res;
+  std::string v_proxy_server;
+  std::string v_proxy_auth_str;
+  std::string v_www_agent = WWW_AGENT;
+  f_curl_upload_data v_data;
+
+  FILE *v_destinationFile;
+  std::string v_local_file;
+  v_local_file = vapp::FS::getUserDir() + "/" + DEFAULT_REPLAYUPLOAD_MSGFILE;
+
+
+  struct curl_httppost *v_post, *v_last;
+
+  vapp::Verbose(std::string("Uploading replay " + p_replayFilename).c_str());
+
+  /* open the file */
+  if( (v_destinationFile = fopen(v_local_file.c_str(), "wb")) == false) {
+    throw vapp::Exception("error : unable to open output file " DEFAULT_REPLAYUPLOAD_MSGFILE);
+  }
+      
+  v_curl = curl_easy_init();
+  if(v_curl == false) {
+    fclose(v_destinationFile);
+    remove(v_local_file.c_str());
+    throw vapp::Exception("error : unable to init curl");	
+  }
+
+  curl_easy_setopt(v_curl, CURLOPT_URL, p_url_to_transfert.c_str());
+
+  v_post = NULL;
+  v_last = NULL;
+  
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "id_room",
+	       CURLFORM_PTRCONTENTS, p_id_room.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "login",
+	       CURLFORM_PTRCONTENTS, p_login.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "password",
+	       CURLFORM_PTRCONTENTS, p_password.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "replay",
+	       CURLFORM_FILE, p_replayFilename.c_str(), CURLFORM_END);
+
+  curl_easy_setopt(v_curl, CURLOPT_HTTPPOST, v_post);
+
+  curl_easy_setopt(v_curl, CURLOPT_TIMEOUT, DEFAULT_TRANSFERT_TIMEOUT);
+  curl_easy_setopt(v_curl, CURLOPT_CONNECTTIMEOUT, DEFAULT_TRANSFERT_CONNECT_TIMEOUT);
+  curl_easy_setopt(v_curl, CURLOPT_WRITEDATA, v_destinationFile);
+  curl_easy_setopt(v_curl, CURLOPT_WRITEFUNCTION, FSWeb::writeData);
+  curl_easy_setopt(v_curl, CURLOPT_USERAGENT,  v_www_agent.c_str());
+  curl_easy_setopt(v_curl, CURLOPT_FAILONERROR, 1);
+
+  /* set proxy settings */
+  if(p_proxy_settings != NULL) {
+    /* v_proxy_server because 
+       after call to 
+       curl_easy_setopt(v_curl, CURLOPT_PROXY, p_proxy_settings->getServer().c_str());
+       result is destroyed on call to curl_easy_perform
+    */
+    v_proxy_server = p_proxy_settings->getServer();
+    v_proxy_auth_str =
+      p_proxy_settings->getAuthentificationUser()
+      + ":"
+      + p_proxy_settings->getAuthentificationPassword();
+
+    if(p_proxy_settings->useDefaultServer() == false) {
+      vapp::Verbose(std::string("set proxy ->" + v_proxy_server).c_str());
+      curl_easy_setopt(v_curl, CURLOPT_PROXY, v_proxy_server.c_str());
+    }
+    
+    if(p_proxy_settings->useDefaultPort() == false) {
+      curl_easy_setopt(v_curl, CURLOPT_PROXYPORT, p_proxy_settings->getPort());
+    }
+    
+    curl_easy_setopt(v_curl, CURLOPT_PROXYTYPE, p_proxy_settings->getType());
+   
+    if(p_proxy_settings->useDefaultAuthentification() == false) {
+      vapp::Verbose(std::string("set proxy authentification ->" + v_proxy_auth_str).c_str());
+      curl_easy_setopt(v_curl, CURLOPT_PROXYUSERPWD,
+		       v_proxy_auth_str.c_str());
+    }
+  }
+  /* ***** */
+
+  if(p_WebApp != NULL) {
+    v_data.v_WebApp = p_WebApp;
+
+    curl_easy_setopt(v_curl, CURLOPT_NOPROGRESS, false);
+    curl_easy_setopt(v_curl, CURLOPT_PROGRESSFUNCTION, FSWeb::f_curl_progress_callback_upload);
+    curl_easy_setopt(v_curl, CURLOPT_PROGRESSDATA, &v_data);
+  }
+
+  v_res = curl_easy_perform(v_curl);
+
+  fclose(v_destinationFile);
+
+  /* CURLE_ABORTED_BY_CALLBACK is not considered as an error */
+  if(v_res != CURLE_ABORTED_BY_CALLBACK) {
+    if(v_res != CURLE_OK) {
+      curl_easy_cleanup(v_curl);
+      remove(v_local_file.c_str());
+      throw vapp::Exception("error : unable to perform curl");	
+    }
+  }
+  curl_easy_cleanup(v_curl);
+
+  /* analyse de la réponse */
+  if(v_res == CURLE_ABORTED_BY_CALLBACK) {
+    p_msg_status = 0;
+    p_msg = "Upload aborted\nYou will be able to validate it later ;-)";
+  } else {
+    uploadReplayAnalyseMsg(v_local_file, p_msg_status, p_msg);
+  }
+
+  remove(v_local_file.c_str());
+}
+
+void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
+				   bool &p_msg_status_ok,
+				   std::string &p_msg) {
+  vapp::XMLDocument v_Xml;
+  TiXmlDocument *v_XmlData;
+  TiXmlElement *v_XmlDataElement;
+  TiXmlElement *v_XmlDataElementMsg;
+  std::string v_success;
+  const char *pc;
+  TiXmlNode * pChild;
+
+  /* open the file */
+  v_Xml.readFromFile(p_filename);   
+  v_XmlData = v_Xml.getLowLevelAccess();
+  
+  if(v_XmlData == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+  
+  /* read the res and msg */
+  v_XmlDataElement = v_XmlData->FirstChildElement("xmoto_uploadReplayResult");
+  if(v_XmlDataElement == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+
+  pc = v_XmlDataElement->Attribute("success");
+  if(pc == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+  v_success = pc;
+  p_msg_status_ok = v_success == "1";
+
+  v_XmlDataElementMsg = v_XmlDataElement->FirstChildElement("message");
+  if(v_XmlDataElementMsg == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+
+  pChild = v_XmlDataElementMsg->FirstChild();
+  if(pChild == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+
+  if(pChild->ToText() == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+
+  pc = pChild->ToText()->Value();
+  if(pc == NULL) {
+    throw vapp::Exception("unable to analyze xml file result");
+  }
+  p_msg = pc;
+}
+
 WebLevel::WebLevel(std::string p_id, std::string p_name, std::string p_url) {
   m_id   = p_id;
   m_name = p_name;
@@ -601,13 +801,13 @@ void WebLevels::extractLevelsToDownloadFromXml() {
   v_webLXmlData = v_webLXml.getLowLevelAccess();
 
   if(v_webLXmlData == NULL) {
-    throw vapp::Exception("error : unable analyse xml level file");
+    throw vapp::Exception("error : unable to analyze xml level file");
   }
 
   v_webLXmlDataElement = v_webLXmlData->FirstChildElement("xmoto_levels");
   
   if(v_webLXmlDataElement == NULL) {
-    throw vapp::Exception("error : unable analyse xml level file");
+    throw vapp::Exception("error : unable to analyze xml level file");
   }
 
   TiXmlElement *pVarElem = v_webLXmlDataElement->FirstChildElement("level");
@@ -666,17 +866,37 @@ void WebLevels::update() {
   extractLevelsToDownloadFromXml();
 }
 
-struct f_curl_download_data {
-  vapp::WWWAppInterface *v_WebApp;
-  int v_nb_files_to_download;
-  int v_nb_files_performed;
-};
+int FSWeb::f_curl_progress_callback_upload(void *clientp,
+					   double dltotal,
+					   double dlnow,
+					   double ultotal,
+					   double ulnow) {
+  f_curl_download_data *data = ((f_curl_download_data*) clientp);
+  float v_percentage;
 
-int FSWeb::f_curl_progress_callback(void *clientp,
-				    double dltotal,
-				    double dlnow,
-				    double ultotal,
-				    double ulnow) {
+  /* cancel if it's wanted */
+  if(data->v_WebApp->isCancelAsSoonAsPossible()) {
+    return 1;
+  }
+
+  /* we can't make trust to the web server information */
+  v_percentage = 0.0;
+  if(ultotal > 0.0) {
+    if((ulnow / ultotal) >= 0.0 && (ulnow / ultotal) <= 1.0) {
+      v_percentage = (ulnow * 100.0) /ultotal;
+    }
+  }
+
+  data->v_WebApp->setTaskProgress(v_percentage);
+
+  return 0;
+}
+
+int FSWeb::f_curl_progress_callback_download(void *clientp,
+					     double dltotal,
+					     double dlnow,
+					     double ultotal,
+					     double ulnow) {
   f_curl_download_data *data = ((f_curl_download_data*) clientp);
   float real_percentage_already_done;
   float real_percentage_of_current_file;
@@ -690,7 +910,7 @@ int FSWeb::f_curl_progress_callback(void *clientp,
     (((float)data->v_nb_files_performed) * 100.0) 
     / ((float)data->v_nb_files_to_download);
 
-  /* we can't make confiance to the web server information */
+  /* we can't make trust to the web server information */
   real_percentage_of_current_file = 0.0;
   if(dltotal > 0.0) {
     if((dlnow / dltotal) >= 0.0 && (dlnow / dltotal) <= 1.0) {
@@ -705,6 +925,19 @@ int FSWeb::f_curl_progress_callback(void *clientp,
   return 0;
 }
 
+bool WebLevels::exists(const std::string p_id) {
+  std::vector<WebLevel*>::iterator it;
+  it = m_webLevels.begin();
+  while(it != m_webLevels.end()) {
+    if((*it)->getId() == p_id) {
+      return true;
+    }
+    it++;
+  }
+
+  return false;
+}
+
 void WebLevels::upgrade() {
   std::vector<WebLevel*>::iterator it;
   f_curl_download_data v_data;
@@ -746,7 +979,7 @@ void WebLevels::upgrade() {
       
       FSWeb::downloadFileBz2(v_destFile,
 			     v_url,
-			     FSWeb::f_curl_progress_callback,
+			     FSWeb::f_curl_progress_callback_download,
 			     &v_data,
 			     m_proxy_settings);
       
@@ -839,13 +1072,13 @@ void WebThemes::extractThemesAvailableFromXml() {
   v_webTXmlData = v_webTXml.getLowLevelAccess();
 
   if(v_webTXmlData == NULL) {
-    throw vapp::Exception("error : unable analyse xml theme file");
+    throw vapp::Exception("error : unable to analyze xml theme file");
   }
 
   v_webTXmlDataElement = v_webTXmlData->FirstChildElement("xmoto_themes");
   
   if(v_webTXmlDataElement == NULL) {
-    throw vapp::Exception("error : unable analyse xml theme file");
+    throw vapp::Exception("error : unable to analyze xml theme file");
   }
 
   TiXmlElement *pVarElem = v_webTXmlDataElement->FirstChildElement("theme");
@@ -884,6 +1117,15 @@ const std::vector<WebTheme*> &WebThemes::getAvailableThemes() {
   return m_availableThemes;
 }
 
+bool WebThemes::isUpgradable(ThemeChoice *p_themeChoice) {
+  for(unsigned int i=0; i<m_availableThemes.size(); i++) {
+    if(m_availableThemes[i]->getName() == p_themeChoice->ThemeName()) {
+      return true;
+    }
+  }
+  return false;
+}
+
 void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
   std::string v_destinationFile;
   std::string v_sourceFile;
@@ -903,29 +1145,30 @@ void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
     }
   }
 
+  if(v_associated_webtheme == NULL) {
+    throw vapp::Exception("error : unable to find this theme on the web");
+  }
+  // theme avaible on the web get it
+
   /* the destination file must be in the user dir */
   if(vapp::FS::isInUserDir(p_themeChoice->ThemeFile())) {
     v_destinationFile = p_themeChoice->ThemeFile();
   }
 
-  // theme avaible on the web get it
-  if(v_associated_webtheme != NULL) {
-
-    if(v_destinationFile == "") {
-      /* determine destination file */
-      v_destinationFile = 
-	vapp::FS::getUserDir() + "/" + THEMES_DIRECTORY + "/" + 
-	vapp::FS::getFileBaseName(v_associated_webtheme->getUrl()) + ".xml";
-    }    
-
-    /* download the theme file */
-    vapp::FS::mkArborescence(v_destinationFile);
-    FSWeb::downloadFileBz2(v_destinationFile,
-			   v_associated_webtheme->getUrl(),
-			   NULL,
-			   NULL,
-			   m_proxy_settings);
-  }
+  if(v_destinationFile == "") {
+    /* determine destination file */
+    v_destinationFile = 
+      vapp::FS::getUserDir() + "/" + THEMES_DIRECTORY + "/" + 
+      vapp::FS::getFileBaseName(v_associated_webtheme->getUrl()) + ".xml";
+  }    
+  
+  /* download the theme file */
+  vapp::FS::mkArborescence(v_destinationFile);
+  FSWeb::downloadFileBz2(v_destinationFile,
+			 v_associated_webtheme->getUrl(),
+			 NULL,
+			 NULL,
+			 m_proxy_settings);
 
   if(v_destinationFile == "") {
     throw vapp::Exception("error : unable to determine theme destination file");
@@ -968,7 +1211,7 @@ void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
 
 	FSWeb::downloadFile(v_destinationFile,
 			    v_sourceFile,
-			    FSWeb::f_curl_progress_callback,
+			    FSWeb::f_curl_progress_callback_download,
 			    &v_data,
 			    m_proxy_settings);
 	
@@ -986,4 +1229,110 @@ void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
   p_themeChoice->setHosted(true);
 }
 
+WebRoomInfos::WebRoomInfos(std::string p_id,
+			   std::string p_name,
+			   std::string p_urlHighscores) {
+  m_id           = p_id;
+  m_name         = p_name;
+  m_urlHighscore = p_urlHighscores;
+}
+
+WebRoomInfos::~WebRoomInfos() {
+}
+
+std::string WebRoomInfos::getId() {
+  return m_id;
+}
+
+std::string WebRoomInfos::getName() {
+  return m_name;
+}
+
+std::string WebRoomInfos::getUrlHighscores() {
+  return m_urlHighscore;
+}
+
+WebRooms::WebRooms(const ProxySettings *p_proxy_settings) {
+  m_proxy_settings = p_proxy_settings;
+}
+
+WebRooms::~WebRooms() {
+  clean();
+}
+
+void WebRooms::update() {
+  FSWeb::downloadFileBz2UsingMd5(getXmlFileName(),
+				 m_rooms_url,
+				 NULL,
+				 NULL,
+				 m_proxy_settings);
+}
+
+std::string WebRooms::getXmlFileName() {
+  return vapp::FS::getUserDir() + "/" + DEFAULT_WEBROOMS_FILENAME;
+}
+
+void WebRooms::upgrade() {
+  clean();
+
+  try {
+    vapp::XMLDocument v_webRXml;
+    TiXmlDocument *v_webRXmlData;
+    TiXmlElement *v_webRXmlDataElement;
+    const char *pc;
+    std::string v_RoomName, v_RoomHighscoreUrl, v_RoomId;
+  
+    v_webRXml.readFromFile(getXmlFileName());
+    v_webRXmlData = v_webRXml.getLowLevelAccess();
+
+    if(v_webRXmlData == NULL) {
+      throw vapp::Exception("error : unable to analyze xml rooms list file");
+    }
+
+    v_webRXmlDataElement = v_webRXmlData->FirstChildElement("xmoto_rooms");
+    
+    if(v_webRXmlDataElement == NULL) {
+      throw vapp::Exception("error : unable to analyze xml rooms list file");
+    }
+    
+    TiXmlElement *pVarElem = v_webRXmlDataElement->FirstChildElement("room");
+    while(pVarElem != NULL) {
+      
+      pc = pVarElem->Attribute("name");
+      if(pc != NULL) {
+	v_RoomName = pc;
+	
+	pc = pVarElem->Attribute("highscores_url");
+	if(pc != NULL) {
+	  v_RoomHighscoreUrl = pc;	 
+	  
+	  pc = pVarElem->Attribute("id");
+	  if(pc != NULL) {
+	    v_RoomId = pc;
+	    m_availableRooms.push_back(new WebRoomInfos(v_RoomId,
+							v_RoomName,
+							v_RoomHighscoreUrl));
+	  }
+	}
+	pVarElem = pVarElem->NextSiblingElement("room");
+      }
+    }
+    
+  } catch(vapp::Exception &e) {
+    /* by default, room is highscore */
+    m_availableRooms.push_back(new WebRoomInfos(DEFAULT_WEBROOM_ID, DEFAULT_WEBROOM_NAME, DEFAULT_WEBHIGHSCORES_URL));
+  }
+}
+
+const std::vector<WebRoomInfos*> &WebRooms::getAvailableRooms() {
+  return m_availableRooms;
+}
+
+void WebRooms::clean() {
+  for(unsigned int i=0;i<m_availableRooms.size();i++) {
+    delete m_availableRooms[i];
+  }    
+  m_availableRooms.clear();
+}
+
 #endif
