@@ -41,13 +41,19 @@ class ThemeChoice;
 #define DEFAULT_WEBHIGHSCORES_URL         "http://xmoto.free.fr/highscores.xml"
 #define DEFAULT_WEBHIGHSCORES_FILENAME    "webhighscores.xml"
 #define DEFAULT_TRANSFERT_TIMEOUT         240
-#define DEFAULT_TRANSFERT_CONNECT_TIMEOUT 15
+#define DEFAULT_TRANSFERT_CONNECT_TIMEOUT 7
 #define DEFAULT_WEBLEVELS_URL             "http://xmoto.free.fr/levels.xml"
 #define DEFAULT_WEBLEVELS_FILENAME        "weblevels.xml"
 #define DEFAULT_WEBLEVELS_DIR             "downloaded"
 #define DEFAULT_WEBTHEMES_URL             "http://xmoto.free.fr/themes.xml"
 #define DEFAULT_WEBTHEMES_FILENAME        "webthemes.xml"
 #define DEFAULT_WEBTHEMES_SPRITESURLBASE  "http://xmoto.free.fr/sprites"
+#define DEFAULT_UPLOADREPLAY_URL          "http://xmoto.free.fr/tools/UploadReplay.php"
+#define DEFAULT_REPLAYUPLOAD_MSGFILE      "UploadReplayMsg.xml"
+#define DEFAULT_WEBROOMS_URL              "http://xmoto.free.fr/rooms.xml"
+#define DEFAULT_WEBROOMS_FILENAME         "webrooms.xml"
+#define DEFAULT_WEBROOM_ID                "1"
+#define DEFAULT_WEBROOM_NAME              "WR"
 
 #define WWW_AGENT ("xmoto-" + vapp::App::getVersionString())
 
@@ -114,42 +120,61 @@ class FSWeb {
  public:
   static void downloadFile(const std::string &p_local_file,
 			   const std::string &p_web_file,
-			   int (*curl_progress_callback)(void *clientp,
-							 double dltotal,
-							 double dlnow,
-							 double ultotal,
-							 double ulnow),
+			   int (*curl_progress_callback_download)(void *clientp,
+								  double dltotal,
+								  double dlnow,
+								  double ultotal,
+								  double ulnow),
 			   void *p_data,
 			   const ProxySettings *p_proxy_settings);
 
   static void downloadFileBz2(const std::string &p_local_file,
 			      const std::string &p_web_file,
-			      int (*curl_progress_callback)(void *clientp,
-							    double dltotal,
-							    double dlnow,
-							    double ultotal,
-							    double ulnow),
+			      int (*curl_progress_callback_download)(void *clientp,
+								     double dltotal,
+								     double dlnow,
+								     double ultotal,
+								     double ulnow),
 			      void *p_data,
 			      const ProxySettings *p_proxy_settings);
 
   static void downloadFileBz2UsingMd5(const std::string &p_local_file,
 				      const std::string &p_web_file,
-				      int (*curl_progress_callback)(void *clientp,
-								    double dltotal,
-								    double dlnow,
-								    double ultotal,
-								    double ulnow),
+				      int (*curl_progress_callback_download)(void *clientp,
+									     double dltotal,
+									     double dlnow,
+									     double ultotal,
+									     double ulnow),
 				      void *p_data,
 				      const ProxySettings *p_proxy_settings);
 
-  static int f_curl_progress_callback(void *clientp,
-				      double dltotal,
-				      double dlnow,
-				      double ultotal,
-				      double ulnow); 
+  static void uploadReplay(std::string p_replayFilename,
+			   std::string p_id_room,
+			   std::string p_login,
+			   std::string p_password,
+			   std::string p_url_to_transfert,
+			   vapp::WWWAppInterface *p_WebApp,
+			   const ProxySettings *p_proxy_settings,
+			   bool &p_msg_status,
+			   std::string &p_msg);
+
+  static int f_curl_progress_callback_upload(void *clientp,
+					     double dltotal,
+					     double dlnow,
+					     double ultotal,
+					     double ulnow);
+
+  static int f_curl_progress_callback_download(void *clientp,
+					       double dltotal,
+					       double dlnow,
+					       double ultotal,
+					       double ulnow);
 
  private:
   static size_t writeData(void *ptr, size_t size, size_t nmemb, FILE *stream);
+  static void   uploadReplayAnalyseMsg(std::string p_filename,
+				       bool &p_msg_status_ok,
+				       std::string &p_msg);
 };
 
 class WebHighscore {
@@ -209,6 +234,46 @@ class WebRoom {
   void cleanHash();
 };
 
+class WebRoomInfos {
+ public:
+  WebRoomInfos(std::string p_id,
+	       std::string p_name,
+	       std::string p_urlHighscores);
+  ~WebRoomInfos();
+
+  std::string getId();
+  std::string getName();
+  std::string getUrlHighscores();
+
+ private:
+  std::string m_id;
+  std::string m_name;
+  std::string m_urlHighscore;
+};
+
+class WebRooms {
+ public:
+  WebRooms(const ProxySettings *p_proxy_settings);
+  ~WebRooms();
+
+  /* check for new rooms */
+  void update(); /* throws exceptions */
+
+  /* fill the list of avaible rooms ; does not required an internet connexion */
+  void upgrade();
+
+  void setURL(const std::string &p_url) {m_rooms_url = p_url;}
+  const std::vector<WebRoomInfos*> &getAvailableRooms();
+
+ private:
+  const ProxySettings *m_proxy_settings;
+  std::vector<WebRoomInfos*> m_availableRooms;
+  std::string m_rooms_url;
+
+  void clean();
+  std::string getXmlFileName();
+};
+
 class WebLevel {
 public:
   WebLevel(std::string p_id, std::string p_name, std::string p_url);
@@ -259,6 +324,8 @@ class WebLevels {
   /* Set URL */
   void setURL(const std::string &p_url) {m_levels_url = p_url;}
 
+  bool exists(const std::string p_id);
+
  private:
   vapp::WWWAppInterface *m_WebLevelApp;
   std::vector<WebLevel*> m_webLevels;
@@ -306,6 +373,7 @@ class WebThemes {
 
   /* download a theme or just update it */
   void upgrade(ThemeChoice *p_themeChoice);
+  bool isUpgradable(ThemeChoice *p_themeChoice);
 
   const std::vector<WebTheme*> &getAvailableThemes();
 
